<?php
/**
 * @name Model_plan
 * @description Classe modèle Plan
 *		Toutes les méthodes d'acces à la DB pour le module plan
 * @author Quentin George <q.george@esia-sa.com>
 * @date 20/01/2022
 */
include_once RACINE_SITE . 'plugin/plan/plan.conf.php';
include_once MVCMODEL;
include_once ABSTRACT_PLAN_CLASS;
include_once PLAN_CLASS;
include_once OBJECT_PLAN_CLASS;

use \Plan\Plan;
use \Plan\ObjectPlan;
use \Plan\PlanException;

class Model_plan extends Model_esia
{
	const PLAN_SORT_PARENT = 0;
	const PLAN_SORT_CHILDREN = 1;
	const PLAN_OBJ_STATE_MAX_LVL = 4; // (5 appels) Nombre maximum d'appels de fonction autorisé en récusif pour le calcul de l'état d'un objet (si l'objet est lié a d'autres plans)
	const PLAN_OBJ_ALERTS_MAX_LVL = 4; // (5 appels) Nombre maximum d'appels de fonction autorisé en récusif pour la récupération des alertes d'un objet (si l'objet est lié a d'autres plans)
	const PLAN_PRIORITY_ACCESS = array('CREATOR', 'WRITE', 'READ'); // Ordre des priorité d'accès au plan/dossier
	const PLAN_PRIORITY_STATES = array('critical', 'warning', 'unknown', 'ok'); // Ordre des priorités d'état
	const PLAN_STATES_IGNORED = array('maintenance', 'acquitte', 'pending'); // Status a considérer comme ok
	/**
	*** Instancie un objet Plan
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id identifiant du plan
	*** @param integer $userId identifiant de l'utilisateur
	*** @param boolean (optionnel) $isObjects si TRUE, récupère aussi les objets liés à ce plan
	*** @param boolean (optionnel) $isStates si TRUE, récupère aussi le status de tous les objets et le status global du plan
	*** @param string (optionnel) $defaultStates si $isStates vaut FALSE l'état de tous les object et du plan seront définis par cette valeur
	*** @return object Plan class
	*** @throws \Plan\PlanException errors preventing to instanciate the plan throw PlanException class and not \EsiaDbException to treat it differently in controller
	**/
	public static function instancePlan($dbconnection, $id, $userId, $isObjects=FALSE, $isStates=FALSE, $defaultStates='unknown')
	{
		if(empty($id) || !is_numeric($id))
		{
			throw new \Plan\PlanException("Plan ID is not numeric");
		}
		if(empty($userId) || !is_numeric($userId))
		{
			throw new \Plan\PlanException("User ID is not numeric");
		}

		$sql = "SELECT plan_id, plan_name, plan_type_id, plan_types.type_name AS plan_type_name, plan_types.type_controller AS plan_type_controller, plan_params, plan_folder_id, plan_folders.folder_name AS plan_folder_name, (SELECT * FROM getPlanUserAccess(:plan_id, :user_id)) AS plan_user_access
				FROM plan_plans
				LEFT JOIN plan_types ON plan_types.type_id = plan_type_id
				LEFT JOIN plan_folders ON plan_folders.folder_id = plan_folder_id
				WHERE plan_name <> '.ACCESS' AND plan_id = :plan_id";

		if($isObjects)
		{
			$sql = "SELECT plan_id, plan_name,plan_type_id, plan_types.type_name AS plan_type_name, plan_types.type_controller AS plan_type_controller, plan_params, plan_folder_id, plan_folders.folder_name AS plan_folder_name, (SELECT * FROM getPlanUserAccess(:plan_id, :user_id)) AS plan_user_access,
					object_id, object_name, object_link_plan_id, object_params, object_type_id, plan_object_types.type_name AS object_type_name, object_icon_id, plan_object_icons.icon_name AS object_icon_name, plan_object_icons.icon_type AS object_icon_type, plan_object_icons.icon_path AS object_icon_path
					FROM plan_plans
					LEFT JOIN plan_types ON plan_types.type_id = plan_type_id
					LEFT JOIN plan_folders ON plan_folders.folder_id = plan_folder_id
					LEFT JOIN plan_objects ON plan_objects.object_on_plan_id = plan_id
					LEFT JOIN plan_object_types ON plan_object_types.type_id = object_type_id
					LEFT JOIN plan_object_icons ON plan_object_icons.icon_id = object_icon_id
					WHERE plan_name <> '.ACCESS' AND plan_id = :plan_id
					ORDER BY object_id";
		}
		$params = array(
			':plan_id' => $id,
			':user_id' => $userId
		);
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		if($query->rowCount() <= 0)
		{
			throw new \Plan\PlanException('Plan does not exist');
		}
		$plan = new \Plan\Plan();
		$planState = 'ok';
		$isFirstRow = TRUE;
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if($isFirstRow)
			{
				$isFirstRow = FALSE;

				$plan->setId($row->plan_id);
				$plan->setName($row->plan_name);
				$plan->setTypeId($row->plan_type_id);
				$plan->setTypeName($row->plan_type_name);
				$plan->setTypeController($row->plan_type_controller);
				$plan->setParams($row->plan_params);
				$plan->setFolderId($row->plan_folder_id);
				$plan->setFolderName($row->plan_folder_name);
				$plan->setCurrentUserAccess($row->plan_user_access);
			}

			if($isObjects && !empty($row->object_id))
			{
				$object = Model_plan::instanceObjectPlanByRowDatabase($row);

				if($isStates)
				{
					// Récupèration de l'état de chaque objet ainsi que l'état global du plan
					$objParams = json_decode($row->object_params, TRUE);
					$state = Model_plan::getObjectState($dbconnection, $userId, $objParams, $row->object_type_name, $row->object_link_plan_id);
					$object->setState($state);
					$planState = Model_plan::getMostImportantState($planState, $state);
				}
				else
				{
					$object->setState($defaultStates);
				}

				$plan->setObject($object);
			}
		}
		if($isStates)
		{
			$plan->setState($planState);
		}
		else
		{
			$plan->setState($defaultStates);
		}
		return $plan;
	}
	/**
	*** Instancie un ou plusieurs objets ObjectPlan
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId id du plan auquel appartienent ces objets
	*** @param array||integer (optionnel) $idsList liste d'identifiants d'objets plan. Si vide, intancie tous les objets du plan
	*** @param boolean (optionnel) $isStates si TRUE, récupère aussi le status des objets
	*** @param string (optionnel) $defaultStates si $isStates vaut FALSE l'état de tous les object et du plan seront définis par cette valeur
	*** @return array liste de ObjectPlan class
	*** @throws \Plan\PlanException errors preventing to instanciate the object throw PlanException class and not \EsiaDbException to treat it differently in controller
	**/
	public static function instanceObjectPlans($dbconnection, $planId, $idsList=array(), $isStates=FALSE, $defaultStates='unknown')
	{
		if(empty($planId) || !is_numeric($planId))
		{
			throw new \Plan\PlanException("Plan ID is not numeric");
		}
		$sql = "SELECT object_id, object_name, object_link_plan_id, object_params, object_type_id, type_name AS object_type_name, object_icon_id, icon_name AS object_icon_name, icon_type AS object_icon_type, icon_path AS object_icon_path
				FROM plan_objects
				LEFT JOIN plan_object_types ON type_id = object_type_id
				LEFT JOIN plan_object_icons ON icon_id = object_icon_id
				WHERE object_on_plan_id = :plan_id";

		$params = array(':plan_id' => $planId);
		if(!empty($idsList))
		{
			if(is_numeric($idsList))
			{
				$sql .= " AND object_id = :object_id";
				$params[':object_id'] = $idsList;
			}
			elseif(is_array($idsList))
			{
				foreach($idsList as $id)
				{
					if(!is_numeric($id))
					{
						throw new \Plan\PlanException('Objects list IDs must be numeric');
					}
				}
				$idsStr = implode(',', $idsList);
				$sql .= " AND object_id IN (" . $idsStr . ")";
			}
			else
			{
				throw new \Plan\PlanException('Object ID must be numeric');
			}
		}

		$sql .= " ORDER BY object_id";
		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		if($query->rowCount() <= 0)
		{
			throw new \Plan\PlanException('Object does not exist on this plan: ' . $planId);
		}
		$objects = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$object = Model_plan::instanceObjectPlanByRowDatabase($row);

			if($isStates)
			{
				// Récupèration de l'état de chaque objet
				$objParams = json_decode($row->object_params, TRUE);
				$state = Model_plan::getObjectState($dbconnection, $userId, $objParams, $row->object_type_name, $row->object_link_plan_id);
				$object->setState($state);
			}
			else
			{
				$object->setState($defaultStates);
			}

			$objects[$object->getId()] = $object;
		}
		return $objects;
	}
	/**
	*** Instancie un objet ObjectPlan depuis une requête DB préallablement effecuée
	*** @param object $row object renvoyer par PDO::FETCH_OBJ
	*** @return object ObjectPlan class
	**/
	private function instanceObjectPlanByRowDatabase($row)
	{
		$object = new \Plan\ObjectPlan();
		$object->setId($row->object_id);
		$object->setName($row->object_name);
		$object->setTypeId($row->object_type_id);
		$object->setTypeName($row->object_type_name);
		$object->setParams($row->object_params);
		$object->setLinkPlanId($row->object_link_plan_id);
		$object->setIconId($row->object_icon_id);
		$object->setIconName($row->object_icon_name);
		$object->setIconType($row->object_icon_type);
		$object->setIconPath($row->object_icon_path);

		return $object;
	}
	/**
	*** Liste tous les plans dont l'utilisateur a accès
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur
	*** @param integer (optionnel) $planId identifiant du plan si on ne veut qu'un seul plan
	*** @param integer (optionnel) $parentId identifiant du dossier dans lequel on doit lister les plans. Si NULL liste tous les plans. Si 0 liste les plans de la racine
	*** @return array liste des plans
	*** @throws \EsiaDbException
	**/
	public static function getPlans($dbconnection, $userId, $planId=NULL, $parentId=NULL)
	{
		if(empty($userId) || !is_numeric($userId))
		{
			throw new \EsiaDbException("User ID is not numeric");
		}

		$sql = "SELECT plan_id, plan_name, plan_params, plan_folder_id, type_id, type_name, type_controller
				FROM plan_plans
				JOIN plan_types ON type_id = plan_type_id
				WHERE plan_name <> '.ACCESS' AND plan_id IN (SELECT plan_access_plan_id FROM plan_users_access WHERE plan_access_user_id = :user_id)";

		$params = array(':user_id' => $userId);

		if(!empty($planId) && is_numeric($planId))
		{
			$sql .= " AND plan_id = :plan_id";
			$params[':plan_id'] = $planId;
		}

		if(is_numeric($parentId))
		{
			if($parentId == 0)
			{
				$sql .= " AND plan_folder_id IS NULL";
			}
			else
			{
				$sql .= " AND plan_folder_id = :folder_id";
				$params[':folder_id'] = $parentId;
			}
		}

		$sql .= " ORDER BY plan_name ASC";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$plans = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->plan_id, $plans))
			{
				$plans[$row->plan_id] = array(
					'id' => $row->plan_id,
					'name' => $row->plan_name,
					'params' => json_decode($row->plan_params, TRUE),
					'folders' => $row->plan_folder_id,
					'type_id' => $row->type_id,
					'type_name' => $row->type_name,
					'type_controller' => $row->type_controller
				);
			}
		}

		return $plans;
	}
	/**
	*** Retourne l'acces le plus prioritaire depuis une requête DB préallablement effecuée entre la nouvelle ligne et la préccédente
	*** @param string $new nouvel acces a comparer
	*** @param string $prev acces préccédement sauvegarder
	*** @return string acces le plus prioritaire
	**/
	private function getMostImportantAccess($new, $prev)
	{
		foreach(Model_plan::PLAN_PRIORITY_ACCESS as $p)
		{
			if($new == $p || $prev == $p)
			{
				return $p;
			}
		}

		return $new; // Par securité si la valeur n'existe pas dans PLAN_PRIORITY_ACCESS
	}
	/**
	*** Retourne l'accès utilisateur à un plan donné le plus prioritaire s'il y en a plusieurs
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId identifiant du plan
	*** @param integer $userId identifiant de l'utilisateur
	*** @return string accès de l'user sur ce plan [CREATOR|WRITE|READ]
	*** @throws \EsiaDbException
	**/
	public static function getPlanUserAccess($dbconnection, $planId, $userId)
	{
		$sql = "SELECT * FROM getPlanUserAccess((
					SELECT plan_id
					FROM plan_plans
					WHERE plan_name <> '.ACCESS' AND plan_id = :plan_id
				), :user_id) AS plan_user_access";

		$params = array(
			':plan_id' => $planId,
			':user_id' => $userId
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$access = $query->fetch(PDO::FETCH_OBJ)->plan_user_access;

		if(empty($access))
		{
			$access = 'NONE';
		}

		return $access;
	}
	/**
	*** Retourne les utilisateurs et leurs droits d'accès accès à un plan donné
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId identifiant du plan
	*** @return array liste des user id par type d'accès [CREATOR|WRITE|READ]
	*** @throws \EsiaDbException
	**/
	public static function getPlanAccessUsersList($dbconnection, $planId)
	{
		$sql = "SELECT plan_access_privileges, plan_access_user_id
				FROM plan_plans
				JOIN plan_users_access ON plan_id = plan_access_plan_id
				WHERE plan_name <> '.ACCESS' AND plan_id = :plan_id";

		$params = array(':plan_id' => $planId);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Plan does not exist');
		}

		$accessByUserId = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(isset($accessByUserId[$row->plan_access_user_id]))
			{
				$accessByUserId[$row->plan_access_user_id] = Model_plan::getMostImportantAccess($row->plan_access_privileges, $accessByUserId[$row->plan_access_user_id]);
			}
			else
			{
				$accessByUserId[$row->plan_access_user_id] = $row->plan_access_privileges;
			}
		}

		$usersListByAccess = array(
			'CREATOR' => array(),
			'WRITE' => array(),
			'READ' => array()
		);

		foreach($accessByUserId as $id => $access)
		{
			$usersListByAccess[$access][] = $id;
		}

		return $usersListByAccess;
	}
	/**
	*** Retourne l'accès utilisateur à un dossier/répertoire donné le plus prioritaire s'il y en a plusieurs
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $folderId identifiant du dossier/répertoire
	*** @param integer $userId identifiant de l'utilisateur
	*** @return string accès de l'user sur ce dossier/répertoire [CREATOR|WRITE|READ]
	*** @throws \EsiaDbException
	**/
	public static function getFolderUserAccess($dbconnection, $folderId, $userId)
	{
		$sql = "SELECT * FROM getPlanUserAccess((
					SELECT plan_id
					FROM plan_plans
					WHERE plan_name = '.ACCESS' AND plan_folder_id = :folder_id
				), :user_id) AS plan_user_access";

		$params = array(
			':folder_id' => $folderId,
			':user_id' => $userId
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$access = $query->fetch(PDO::FETCH_OBJ)->plan_user_access;

		if(empty($access))
		{
			$access = 'NONE';
		}

		return $access;
	}
	/**
	*** Retourne les utilisateurs et leurs droits d'accès accès à un répertoire/dossier
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $folderId identifiant du dossier/répertoire
	*** @return array liste des user id par type d'accès [CREATOR|WRITE|READ]
	*** @throws \EsiaDbException
	**/
	public static function getFolderAccessUsersList($dbconnection, $folderId)
	{
		$sql = "SELECT plan_access_privileges, plan_access_user_id
				FROM plan_plans
				JOIN plan_users_access ON plan_id = plan_access_plan_id
				WHERE plan_name = '.ACCESS' AND plan_folder_id = :folder_id";

		$params = array(':folder_id' => $folderId);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Folder does not exist');
		}

		$accessByUserId = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(isset($accessByUserId[$row->plan_access_user_id]))
			{
				$accessByUserId[$row->plan_access_user_id] = Model_plan::getMostImportantAccess($row->plan_access_privileges, $accessByUserId[$row->plan_access_user_id]);
			}
			else
			{
				$accessByUserId[$row->plan_access_user_id] = $row->plan_access_privileges;
			}
		}

		$usersListByAccess = array(
			'CREATOR' => array(),
			'WRITE' => array(),
			'READ' => array()
		);

		foreach($accessByUserId as $id => $access)
		{
			$usersListByAccess[$access][] = $id;
		}

		return $usersListByAccess;
	}
	/**
	*** Ajoute des accès utilisateurs à un plan donné
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId identifiant du plan
	*** @param array $userIds identifiant des utilisateurs
	*** @param string $access accès de l'user sur ce plan [CREATOR|WRITE|READ]
	*** @throws \EsiaDbException
	**/
	public static function addPlanUsersAccess($dbconnection, $planId, $userIds, $access)
	{
		$sql = "INSERT INTO plan_users_access(plan_access_user_id, plan_access_plan_id, plan_access_privileges)
				VALUES (:user_id, :plan_id, :access)";

		$params = array(
			':plan_id' => $planId,
			':access' => strtoupper($access)
		);

		$query = $dbconnection->prepare($sql);

		foreach($userIds as $id)
		{
			$params[':user_id'] = $id;
			$query = Model_esia::executeStatement($query, $params);
		}
	}
	/**
	*** Update les accès d'un plan par type d'accès
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur courrant pour ne pas lui supprimer ses accès
	*** @param integer $planId identifiant du plan
	*** @param array $userIds identifiant des utilisateurs sur lequels ajouter ou garder des accès
	*** @param string $access accès de l'user sur ce plan [CREATOR|WRITE|READ]
	*** @throws \EsiaDbException
	**/
	public static function updatePlanUsersAccess($dbconnection, $userId, $planId, $userIds, $access)
	{
		if(empty($userId) || !is_numeric($userId))
		{
			throw new \EsiaDbException("User ID is not numeric");
		}

		$sql = "DELETE FROM plan_users_access WHERE plan_access_plan_id = :plan_id AND plan_access_privileges = :access AND plan_access_user_id <> :user_id";

		$params = array(
			':plan_id' => $planId,
			':access' => strtoupper($access),
			'user_id' => $userId
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		Model_plan::addPlanUsersAccess($dbconnection, $planId, $userIds, $access);
	}
	/**
	*** Récupère le plan_id d'un dossier
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $folderId identifiant du dossier
	*** @return integer identifiant plan_id du dossier
	*** @throws \EsiaDbException
	**/
	public static function getFolderPlanId($dbconnection, $folderId)
	{
		if(empty($folderId) || !is_numeric($folderId))
		{
			throw new \EsiaDbException('Folder ID is required to fetch the folder plan_id');
		}

		$sql = "SELECT plan_id FROM plan_plans WHERE plan_name = '.ACCESS' AND plan_folder_id = :folder_id";

		$params = array(':folder_id' => $folderId);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
		$row = $query->fetch(PDO::FETCH_OBJ);

		return $row->plan_id;
	}
	/**
	*** Ajoute des accès utilisateurs à un dossier
	*** @param PDO $dbconnection un objet PDO
	*** @param integer (optionnel) $folderPlanId identifiant (plan_id) du dossier dans la table plan_plans
	*** @param array $userIds identifiant des utilisateurs
	*** @param string $access accès de l'user sur ce plan [CREATOR|WRITE|READ]
	*** @param integer (optionnel) $folderId identifiant du dossier (nécessaire pour récupérer $folderPlanId s'il n'est pas fourni)
	*** @throws \EsiaDbException
	**/
	public static function addFolderUsersAccess($dbconnection, $folderPlanId=NULL, $userIds, $access, $folderId=NULL)
	{
		if(empty($folderPlanId) || !is_numeric($folderPlanId))
		{
			$folderPlanId = Model_plan::getFolderPlanId($dbconnection, $folderId);
		}
		Model_plan::addPlanUsersAccess($dbconnection, $folderPlanId, $userIds, $access);
	}
	/**
	*** Update les accès d'un dossier par type d'accès
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur
	*** @param integer (optionnel) $folderPlanId identifiant (plan_id) du dossier dans la table plan_plans
	*** @param array $userIds identifiant des utilisateurs
	*** @param string $access accès de l'user sur ce dossier [CREATOR|WRITE|READ]
	*** @param integer (optionnel) $folderId identifiant du dossier (nécessaire pour récupérer $folderPlanId s'il n'est pas fourni)
	*** @throws \EsiaDbException
	**/
	public static function updateFolderUsersAccess($dbconnection, $userId, $folderPlanId=NULL, $userIds, $access, $folderId=NULL)
	{
		if(empty($folderPlanId) || !is_numeric($folderPlanId))
		{
			$folderPlanId = Model_plan::getFolderPlanId($dbconnection, $folderId);
		}
		Model_plan::updatePlanUsersAccess($dbconnection, $userId, $folderPlanId, $userIds, $access);
	}
	/**
	*** Ajouter un plan
	*** @param PDO $dbconnection un objet PDO
	*** @param string $name le nom du plan
	*** @param integer $typeId l'identifiant du type de plan à ajouter
	*** @param integer (optionnel) $userId identifiant de l'utilisateur créateur
	*** @param integer (optionnel) $folderId identifiant du répertoire (0 peut être passer pour racine qui vaut NULL en DB)
	*** @param json (optionnel) $planParams paramètres du plan au format JSON
	*** @return integer id du nouveau plan en DB
	*** @throws \EsiaDbException
	**/
	public static function addPlan($dbconnection, $name, $typeId, $userId=NULL, $folderId=NULL, $planParams=NULL)
	{
		$sqlInsertColmns = "plan_id, plan_name, plan_type_id";
		$sqlInsertValues = "DEFAULT, :name, :type_id";

		$params = array(
			':name' => $name,
			':type_id' => $typeId
		);

		if(!empty($planParams) && is_string($planParams))
		{
			// is_string to check if it's JSON encoded
			$sqlInsertColmns .= ", plan_params";
			$sqlInsertValues .= ", :plan_params";
			$params[':plan_params'] = $planParams;
		}
		else
		{
			$sqlInsertColmns .= ", plan_params";
			$sqlInsertValues .= ", (SELECT type_default_params FROM plan_types WHERE type_id = :type_id)";
		}

		if(!empty($folderId) && is_numeric($folderId))
		{
			$sqlInsertColmns .= ", plan_folder_id";
			$sqlInsertValues .= ", :folder_id";
			$params[':folder_id'] = $folderId;
		}

		$sql = "INSERT INTO plan_plans (" . $sqlInsertColmns . ")
				VALUES(" . $sqlInsertValues . ")
				RETURNING plan_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$planId = $query->fetch(PDO::FETCH_OBJ)->plan_id;

		if(!empty($userId) && is_numeric($userId))
		{
			Model_plan::addPlanUsersAccess($dbconnection, $planId, array($userId), 'CREATOR');
		}

		return $planId;
	}
	/**
	*** Update un plan
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur
	*** @param integer $id l'identifiant du plan a updater
	*** @param string $name le nom du plan
	*** @param json $planParams paramètres du plan au format JSON
	*** @param integer (optionnel) $folderId identifiant du répertoire (0 peut être passer pour racine qui vaut NULL en DB)
	*** @param array (optionnel) $accessUsersList liste d'id utilisateurs par types d'accesspar types d'access sur ce plan. 'READ' => array($userIdsRead), 'WRITE' => array($userIdsWrite)
	*** @throws \EsiaDbException
	**/
	public static function updatePlan($dbconnection, $userId, $id, $name, $planParams, $folderId=NULL, $accessUsersList=array())
	{
		if(empty($userId) || !is_numeric($userId))
		{
			throw new \EsiaDbException("User ID is not numeric");
		}
		if(!is_string($planParams))
		{
			throw new \EsiaDbException('Plan parameters must be JSON encoded');
		}

		$params = array(
			':id' => $id,
			':name' => $name,
			':plan_params' => $planParams
		);

		if(empty($folderId) || !is_numeric($folderId))
		{
			$sql = "UPDATE plan_plans SET plan_name = :name, plan_params = :plan_params, plan_folder_id = NULL";
		}
		else
		{
			$sql = "UPDATE plan_plans SET plan_name = :name, plan_params = :plan_params, plan_folder_id = :folder_id";

			$params[':folder_id'] = $folderId;
		}

		$sql .= " WHERE plan_name <> '.ACCESS' AND plan_id = :id
				RETURNING plan_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Plan does not exist');
		}

		// Update user access
		if(isset($accessUsersList['READ']))
		{
			Model_plan::updatePlanUsersAccess($dbconnection, $userId, $id, $accessUsersList['READ'], 'READ');
		}
		if(isset($accessUsersList['WRITE']))
		{
			Model_plan::updatePlanUsersAccess($dbconnection, $userId, $id, $accessUsersList['WRITE'], 'WRITE');
		}
	}
	/**
	*** Supprime un plan
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id l'identifiant du plan a supprimer
	*** @throws \EsiaDbException
	**/
	public static function deletePlan($dbconnection, $id)
	{
		if(empty($id) || !is_numeric($id))
		{
			throw new \EsiaDbException("Plan ID is not numeric");
		}
		$params = array(':plan_id' => $id);

		// First, update objects from other plans that reference this plan
		$query = $dbconnection->prepare("UPDATE plan_objects SET object_link_plan_id = NULL WHERE object_link_plan_id = :plan_id");
		$query = Model_esia::executeStatement($query, $params);
		// Delete all objects on this plan
		$query = $dbconnection->prepare("DELETE FROM plan_objects WHERE object_on_plan_id = :plan_id");
		$query = Model_esia::executeStatement($query, $params);
		// Delete users access
		$query = $dbconnection->prepare("DELETE FROM plan_users_access WHERE plan_access_plan_id = :plan_id");
		$query = Model_esia::executeStatement($query, $params);
		// Delete the plan
		$query = $dbconnection->prepare("DELETE FROM plan_plans WHERE plan_id = :plan_id");
		$query = Model_esia::executeStatement($query, $params);
	}
	/**
	*** Déplace un plan dans un répertoire/dossier (Autre que la racine. Utilisé uniquement pour un déplacement gliser/deposer)
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId l'identifiant du plan a déplacer
	*** @param integer $folderId l'identifiant du dossier/répertoire dans lequel déplacer
	*** @throws \EsiaDbException
	**/
	public static function movePlan($dbconnection, $planId, $folderId)
	{
		if(is_numeric($folderId) && is_numeric($planId))
		{
			$query = $dbconnection->prepare("UPDATE plan_plans SET plan_folder_id = :folder_id WHERE plan_id = :plan_id");
			$query = Model_esia::executeStatement($query, array(':plan_id' => $planId, ':folder_id' => $folderId));
		}
		else
		{
			throw new \EsiaDbException('Destination and element ids must be numeric');
		}
	}
	/**
	*** Déplace un dossier dans un autre répertoire/dossier (Autre que la racine. Utilisé uniquement pour un déplacement gliser/deposer)
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $folderId l'identifiant du dossier a déplacer
	*** @param integer $parentId l'identifiant du dossier/répertoire dans lequel déplacer
	*** @throws \EsiaDbException
	**/
	public static function moveFolder($dbconnection, $folderId, $parentId)
	{
		if(is_numeric($parentId) && is_numeric($folderId))
		{
			$query = $dbconnection->prepare("UPDATE plan_folders SET folder_parent_id = :parent_id WHERE folder_id = :folder_id");
			$query = Model_esia::executeStatement($query, array(':folder_id' => $folderId, ':parent_id' => $parentId));
		}
		else
		{
			throw new \EsiaDbException('Destination and element ids must be numeric');
		}
	}
	/**
	*** Récupére un dossier ou la liste des dossiers d'un utilisateur
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur
	*** @param array||integer (optionnel) $folderIds integer ou tableau d'integer identifiants des dossier en DB. Si $parentId & $folderIds sont vides, liste tous les dossier dont l'utilisateur a les accès
	*** @param integer (optionnel) $parentId identifiant du dossier dans lequel on doit regarder. Si 0 liste les dossiers de la racine. Si $parentId & $folderIds sont vides, liste tous les dossier dont l'utilisateur a les accès
	*** @param boolean (optionnel) $isOnlyWrite si TRUE retourne uniquement les dossier dont l'user a les access WRITE ou CREATOR
	*** @return array liste des dossiers
	*** @throws \EsiaDbException
	**/
	public static function getFolders($dbconnection, $userId, $folderIds=NULL, $parentId=NULL, $isOnlyWrite=FALSE)
	{
		if(empty($userId) || !is_numeric($userId))
		{
			throw new \EsiaDbException("User ID is not numeric");
		}

		$sql = "SELECT folder_id, folder_name, folder_parent_id
				FROM plan_folders
				JOIN plan_plans ON plan_folder_id = folder_id
				JOIN plan_users_access ON plan_access_plan_id = plan_id
				WHERE plan_name = '.ACCESS' AND plan_access_user_id = :user_id";

		$params = array(':user_id' => $userId);

		if(!empty($folderIds))
		{
			if(is_numeric($folderIds))
			{
				$sql .= " AND folder_id = :folder_id";
				$params['folder_id'] = $folderIds;
			}
			elseif(is_array($folderIds))
			{
				foreach($folderIds as $id)
				{
					if(!is_numeric($id))
					{
						throw new \EsiaDbException('Folders list IDs must be numeric');
					}
				}
				$folderIds = implode(',', $folderIds);

				$sql .= " AND folder_id IN (" . $folderIds . ")";
			}
			else
			{
				throw new \EsiaDbException('Folder ID must be numeric');
			}
		}

		if(is_numeric($parentId))
		{
			if($parentId == 0)
			{
				$sql .= " AND folder_parent_id IS NULL";
			}
			else
			{
				$sql .= " AND folder_parent_id = :folder_id";
				$params[':folder_id'] = $parentId;
			}
		}

		if($isOnlyWrite)
		{
			$sql .= " AND (plan_access_privileges = 'CREATOR' OR plan_access_privileges = 'WRITE')";
		}

		$sql .= " ORDER BY folder_parent_id DESC, folder_name";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$folders = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->folder_id, $folders))
			{
				$folders[$row->folder_id] = array(
					'id' => $row->folder_id,
					'name' => $row->folder_name,
					'parent' => $row->folder_parent_id
				);
			}
		}

		return $folders;
	}
	/**
	*** Récupére la liste des dossiers parents ou enfants d'un dossier ou d'une liste de dossiers dont l'utilisateur a access
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur
	*** @param array||integer $folderIds integer ou tableau d'integer identifiants des dossier a partir desquel rechercher les enfants ou parents
	*** @param integer (optionnel) $treeDirection PLAN_SORT_PARENT (0) remonte l'arbre vers la racine. PLAN_SORT_CHILDREN (1) descend l'arbre vers le dernier enfant
	*** @return array liste des dossiers
	*** @throws \EsiaDbException
	**/
	public static function getFoldersTree($dbconnection, $userId, $folderIds, $treeDirection=Model_plan::PLAN_SORT_PARENT)
	{
		if(empty($userId) || !is_numeric($userId))
		{
			throw new \EsiaDbException("User ID is not numeric");
		}

		$dbFunction = 'getallParentPlanFolder';
		if($treeDirection == Model_Plan::PLAN_SORT_CHILDREN)
		{
			$dbFunction = 'getallchildplanfolder';
		}

		$params = array();
		if(is_numeric($folderIds))
		{
			$sql = "SELECT folder_id, folder_name, folder_parent_id
					FROM (
						SELECT DISTINCT fold_id AS folder_id, fold_name AS folder_name, fold_parent_id AS folder_parent_id
						FROM " . $dbFunction . "(:folder_id::integer)
					) AS T2";
			$params[':folder_id'] = $folderIds;
		}
		elseif(is_array($folderIds))
		{
			foreach($folderIds as $id)
			{
				if(!is_numeric($id))
				{
					throw new \EsiaDbException('Folders list IDs must be numeric');
				}
			}
			$idsStr = implode(',', $folderIds);

			$sql = "SELECT folder_id, folder_name, folder_parent_id
					FROM (
						SELECT DISTINCT fold_id AS folder_id, fold_name AS folder_name, fold_parent_id AS folder_parent_id
						FROM " . $dbFunction . "('{ $idsStr }'::int[])
					) AS T2";
		}
		else
		{
			throw new \EsiaDbException('Folder ID must be numeric');
		}

		$sql .= " JOIN plan_plans ON plan_folder_id = folder_id
				JOIN plan_users_access ON plan_access_plan_id = plan_id
				WHERE plan_name = '.ACCESS' AND plan_access_user_id = :user_id
				ORDER BY folder_parent_id DESC, folder_name";

		$params[':user_id'] = $userId;

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Folder does not exist or the user cannot access to it');
		}

		$folders = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->folder_id, $folders))
			{
				$folders[$row->folder_id] = array(
					'id' => $row->folder_id,
					'name' => $row->folder_name,
					'parent' => $row->folder_parent_id
				);
			}
		}

		return $folders;
	}
	/**
	*** Ajouter un plan
	*** @param PDO $dbconnection un objet PDO
	*** @param string $name le nom du dossier
	*** @param integer (optionnel) $userId identifiant de l'utilisateur créateur
	*** @param integer (optionnel) $parentId identifiant du répertoire parent (0 peut être passer pour racine qui vaut NULL en DB)
	*** @return integer id du nouveau dossier en DB (folder_id)
	*** @throws \EsiaDbException
	**/
	public static function addFolder($dbconnection, $name, $userId=NULL, $parentId=NULL)
	{
		// plan_folders table
		$params = array(':name' => $name);

		if(empty($parentId) || !is_numeric($parentId))
		{
			$sql = "INSERT INTO plan_folders(folder_id, folder_name)
					VALUES (DEFAULT, :name)
					RETURNING folder_id";
		}
		else
		{
			$sql = "INSERT INTO plan_folders(folder_id, folder_name, folder_parent_id)
					VALUES (DEFAULT, :name, :parent_id)
					RETURNING folder_id";

			$params[':parent_id'] = $parentId;
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$folderId = $query->fetch(PDO::FETCH_OBJ)->folder_id;

		// plan_plans table
		$sql = "INSERT INTO plan_plans(plan_id, plan_name, plan_folder_id)
				VALUES (DEFAULT, '.ACCESS', :folder_id)
				RETURNING plan_id";

		$params = array(':folder_id' => $folderId);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$planId = $query->fetch(PDO::FETCH_OBJ)->plan_id;

		// plan_user_access table
		if(!empty($userId) && is_numeric($userId))
		{
			Model_plan::addFolderUsersAccess($dbconnection, $planId, array($userId), 'CREATOR', NULL);
		}

		return $folderId;
	}
	/**
	*** Modifie un répertoire/dossier
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur
	*** @param integer $id l'identifiant du dossier a updater
	*** @param string $name le nom du dossier
	*** @param integer (optionnel) $parentId identifiant du répertoire parent (0 peut être passer pour racine qui vaut NULL en DB)
	*** @param array (optionnel) $accessUsersList liste d'id utilisateurs par types d'access sur ce dossier. 'READ' => array($userIdsRead), 'WRITE' => array($userIdsWrite)
	*** @throws \EsiaDbException
	**/
	public static function updateFolder($dbconnection, $userId, $id, $name, $parentId=NULL, $accessUsersList=array())
	{
		if(empty($userId) || !is_numeric($userId))
		{
			throw new \EsiaDbException("User ID is not numeric");
		}

		$params = array(
			':id' => $id,
			':name' => $name
		);

		if(empty($parentId) || !is_numeric($parentId))
		{
			$sql = "UPDATE plan_folders SET folder_name = :name, folder_parent_id = NULL";
		}
		else
		{
			$sql = "UPDATE plan_folders SET folder_name = :name, folder_parent_id = :parent_id";

			$params[':parent_id'] = $parentId;
		}

		$sql .= " WHERE folder_id = :id
				RETURNING folder_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Folder does not exist');
		}

		// Update user access
		if(isset($accessUsersList['READ']))
		{
			Model_plan::updateFolderUsersAccess($dbconnection, $userId, NULL, $accessUsersList['READ'], 'READ', $id);
		}
		if(isset($accessUsersList['WRITE']))
		{
			Model_plan::updateFolderUsersAccess($dbconnection, $userId, NULL, $accessUsersList['WRITE'], 'WRITE', $id);
		}
	}
	/**
	*** Supprime un répertoire/dossier
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id l'identifiant du dossier a supprimer
	*** @param boolean (optionnel) $isDelContent si à TRUE supprime également tout le contenu répertoire et plan en dessous
	*** @throws \EsiaDbException
	**/
	public static function deleteFolder($dbconnection, $id, $isDelContent=FALSE)
	{
		if(empty($id) || !is_numeric($id))
		{
			throw new \EsiaDbException("Folder ID is not numeric");
		}

		$params = array(':folder_id' => $id);

		if($isDelContent == TRUE)
		{
			// Get all children folders ID
			$query = $dbconnection->prepare("SELECT fold_id FROM getallchildplanfolder(:folder_id::integer)");
			$query = Model_esia::executeStatement($query, $params);
			$list = $query->fetchAll(PDO::FETCH_COLUMN);
			$list = implode(',', $list);

			// Remove access & objects from plans & folders which are into the folders list
			$sql = "SELECT plan_id FROM plan_plans WHERE plan_folder_id IN (" . $list . ")";
			$query = $dbconnection->prepare("DELETE FROM plan_users_access WHERE plan_access_plan_id IN ($sql)");
			$query = Model_esia::executeStatement($query);
			$query = $dbconnection->prepare("DELETE FROM plan_objects WHERE object_on_plan_id IN ($sql)");
			$query = Model_esia::executeStatement($query);
			// Delete plans & folders
			$query = $dbconnection->prepare("DELETE FROM plan_plans WHERE plan_folder_id IN (" . $list . ")");
			$query = Model_esia::executeStatement($query);
			$query = $dbconnection->prepare("DELETE FROM plan_folders WHERE folder_id IN (" . $list . ")");
			$query = Model_esia::executeStatement($query);
		}
		else
		{
			// Move the folder content to the root folder
			$query = $dbconnection->prepare("UPDATE plan_plans SET plan_folder_id = NULL WHERE plan_folder_id = :folder_id AND plan_name <> '.ACCESS'");
			$query = Model_esia::executeStatement($query, $params);
			$query = $dbconnection->prepare("UPDATE plan_folders SET folder_parent_id = NULL WHERE folder_parent_id = :folder_id");
			$query = Model_esia::executeStatement($query, $params);

			// Delete the folder & it's users access
			$sql = "DELETE FROM plan_users_access
					WHERE plan_access_plan_id IN(SELECT plan_id FROM plan_plans WHERE plan_folder_id=:folder_id AND plan_name='.ACCESS')";
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query, $params);
			$query = $dbconnection->prepare("DELETE FROM plan_plans WHERE plan_folder_id = :folder_id AND plan_name = '.ACCESS';");
			$query = Model_esia::executeStatement($query, $params);
			$query = $dbconnection->prepare("DELETE FROM plan_folders WHERE folder_id = :folder_id");
			$query = Model_esia::executeStatement($query, $params);
		}
	}
	/**
	*** Récupère les types de plans
	*** @param PDO $dbconnection un objet PDO
	*** @return array liste de tous les types de plans
	*** @throws \EsiaDbException
	**/
	public static function getPlansTypes($dbconnection)
	{
		$sql = "SELECT type_id, type_name, type_controller FROM plan_types
				ORDER BY type_name ASC";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query);

		$types = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$types[] = array(
				'id' => $row->type_id,
				'name' => $row->type_name,
				'controller' => $row->type_controller
			);
		}

		return $types;
	}
	/**
	*** Ajouter un objet sur un plan
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId l'identifiant du plan sur lequel ajouter l'objet
	*** @param string $name le nom de l'objet
	*** @param integer||string $type l'identifiant ou nom du type d'objet à ajouter
	*** @param json $objParams paramètres de l'objet au format JSON
	*** @param integer (optionnel) $icon l'identifiant de l'icone de l'objet à ajouter
	*** @param integer (optionnel) $planLinkedId identifiant du plan auquel l'objet fait référence
	*** @return integer id du nouvel objet en DB
	*** @throws \EsiaDbException
	**/
	public static function addObject($dbconnection, $planId, $name, $type, $objParams, $icon=NULL, $planLinkedId=NULL)
	{
		if(empty($planId) || !is_numeric($planId))
		{
			throw new \EsiaDbException("Plan ID is not numeric");
		}
		if(!is_string($objParams))
		{
			throw new \EsiaDbException('Object parameters must be JSON encoded');
		}

		$params = array(
			':name' => $name,
			':plan_id' => $planId,
			':obj_params' => $objParams
		);

		// Set sql request VALUES string parts for data which can be different
		$sqlValueStr = array(
			'type' => 'NULL',
			'icon' => 'NULL',
			'link_plan' => 'NULL'
		);

		if(is_numeric($type))
		{
			$sqlValueStr['type'] = ":type";
			$params[':type'] = $type;
		}
		elseif(is_string($type))
		{
			$sqlValueStr['type'] = "(SELECT type_id FROM plan_object_types WHERE type_name = :type)";
			$params[':type'] = $type;
		}
		else
		{
			throw new \EsiaDbException('Object type must be type ID or type name');
		}

		if(is_numeric($icon))
		{
			$sqlValueStr['icon'] = ":icon";
			$params[':icon'] = $icon;
		}
		if(is_numeric($planLinkedId))
		{
			$sqlValueStr['link_plan'] = ":link_plan";
			$params[':link_plan'] = $planLinkedId;
		}

		$sql = "INSERT INTO plan_objects (object_id, object_name, object_on_plan_id, object_link_plan_id, object_icon_id, object_type_id, object_params)
				VALUES(DEFAULT, :name, :plan_id, " . $sqlValueStr['link_plan'] . ", " . $sqlValueStr['icon'] . ", " . $sqlValueStr['type'] . ", :obj_params)
				RETURNING object_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$objectId = $query->fetch(PDO::FETCH_OBJ)->object_id;

		return $objectId;
	}
	/**
	*** Modifie un objet sur un plan
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id identifiant de l'objet à modifier
	*** @param integer $planId l'identifiant du plan sur lequel se trouve l'objet
	*** @param string $name le nom de l'objet
	*** @param json $objParams paramètres de l'objet au format JSON
	*** @param integer (optionnel) $icon l'identifiant de l'icone de l'objet (0 peut être passer pour remplacer par NULL en DB)
	*** @param integer (optionnel) $planLinkedId identifiant du plan auquel l'objet fait référence (0 peut être passer pour remplacer par NULL en DB)
	*** @throws \EsiaDbException
	**/
	public static function updateObject($dbconnection, $id, $planId, $name, $objParams, $icon=NULL, $planLinkedId=NULL)
	{
		if(empty($id) || !is_numeric($id))
		{
			throw new \EsiaDbException("Object ID is not numeric");
		}
		if(empty($planId) || !is_numeric($planId))
		{
			throw new \EsiaDbException("Plan ID is not numeric");
		}
		if(!is_string($objParams))
		{
			throw new \EsiaDbException('Object parameters must be JSON encoded');
		}

		$sql = "UPDATE plan_objects SET object_name = :name, object_params = :obj_params";

		$params = array(
			':id' => $id,
			':name' => $name,
			':plan_id' => $planId,
			':obj_params' => $objParams
		);

		if(is_numeric($icon))
		{
			if($icon == 0)
			{
				$sql .= ", object_icon_id = NULL";
			}
			else
			{
				$sql .= ", object_icon_id = :icon";
				$params[':icon'] = $icon;
			}
		}

		if(is_numeric($planLinkedId))
		{
			if($planLinkedId == 0)
			{
				$sql .= ", object_link_plan_id = NULL";
			}
			else
			{
				$sql .= ", object_link_plan_id = :link_plan";
				$params[':link_plan'] = $planLinkedId;
			}
		}

		$sql .= " WHERE object_id = :id AND object_on_plan_id = :plan_id
				RETURNING object_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Object does not exist on this plan: ' . $planId);
		}
	}
	/**
	*** Supprime un ou plusieurs objets
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId l'identifiant du plan sur lequel se trouve l'objet
	*** @param array||integer (optionnel) $idsList liste d'identifiants d'objets plan a supprimer. Si vide on supprime tous les objets du plan
	*** @throws \EsiaDbException
	**/
	public static function deleteObjects($dbconnection, $planId, $idsList)
	{
		if(empty($planId) || !is_numeric($planId))
		{
			throw new \EsiaDbException("Plan ID is not numeric");
		}

		$sql = "DELETE FROM plan_objects WHERE object_on_plan_id = :plan_id";

		$params = array(':plan_id' => $planId);

		if(!empty($idsList))
		{
			if(is_numeric($idsList))
			{
				$sql .= " AND object_id = :object_id";
				$params[':object_id'] = $idsList;
				// Delete object links
				Model_plan::deleteObjectLinks($dbconnection, $planId, $idsList);
			}
			elseif(is_array($idsList))
			{
				foreach($idsList as $id)
				{
					if(is_numeric($id))
					{
						// Delete object links
						Model_plan::deleteObjectLinks($dbconnection, $planId, $id);
					}
					else
					{
						throw new \EsiaDbException('Objects list IDs must be numeric');
					}
				}
				$idsStr = implode(',', $idsList);
				$sql .= " AND object_id IN (" . $idsStr . ")";
			}
			else
			{
				throw new \EsiaDbException('Object ID must be numeric');
			}
		}

		$sql .= " RETURNING object_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Object does not exist on this plan: ' . $planId);
		}
	}
	/**
	*** Supprime les objets liens liés a un objet du plan
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId l'identifiant du plan sur lequel se trouve l'objet
	*** @param integer $linkedObjId identifiant de l'objet à partir duquel on doit supprimer les liens
	*** @throws \EsiaDbException
	**/
	public static function deleteObjectLinks($dbconnection, $planId, $linkedObjId)
	{
		$sql = "DELETE FROM plan_objects
				WHERE object_name ILIKE :name
				AND object_on_plan_id = :plan_id
				AND object_type_id = (SELECT type_id FROM plan_object_types WHERE type_name = 'link')";

		$params = array(
			':plan_id' => $planId,
			':name' => '%' . $linkedObjId . 'object%'
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
	}
	/**
	*** Récupère le nom des plans qui contiennent un objet qui utilise une icone. Dans le but de savoir si l'icone peut être supprimée
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $iconId l'identifiant de l'icône
	*** @return array liste de nom de plans
	*** @throws \EsiaDbException
	**/
	public static function getPlansUsingIcon($dbconnection, $iconId)
	{
		$sql = "SELECT plan_name
				FROM plan_plans
				WHERE plan_id IN (SELECT object_on_plan_id FROM plan_objects WHERE object_icon_id = :icon_id)";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, array(':icon_id' => $iconId));

		$plans = $query->fetchAll(PDO::FETCH_COLUMN);

		return $plans;
	}
	/**
	*** Récupère les icones d'objets
	*** @param PDO $dbconnection un objet PDO
	*** @param integer (optionnel) $id l'identifiant de l'icône si on ne veut en récupéré qu'une
	*** @param string (optionnel) $name le nom de l'icône que l'on veut récupéré
	*** @return array l'icone dont le nom ou l'id correspond aux paramètres $id ou $name. Ou la liste complète des icones d'objets
	*** @throws \EsiaDbException
	**/
	public static function getIcons($dbconnection, $id=NULL, $name='')
	{
		$sql = "SELECT icon_id, icon_name, icon_path, icon_type, icon_group_id
				FROM plan_object_icons";

		$params = array();
		if(!empty($id) && is_numeric($id))
		{
			$sql .= " WHERE icon_id = :id";
			$params[':id'] = $id;
		}
		elseif(!empty($name) && is_string($name))
		{
			$sql .= " WHERE icon_name = :name";
			$params[':name'] = $name;
		}

		$sql .= " ORDER BY icon_name";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$icons = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$icons[] = array(
				'id' => $row->icon_id,
				'name' => $row->icon_name,
				'path' => $row->icon_path,
				'type' => $row->icon_type,
				'group' => $row->icon_group_id
			);
		}

		return $icons;
	}
	/**
	*** Récupère les groupes d'icones
	*** @param PDO $dbconnection un objet PDO
	*** @param integer (optionnel) $id l'identifiant du groupe d'icône si on ne veut en récupéré qu'un
	*** @return array liste de groups d'icones
	*** @throws \EsiaDbException
	**/
	public static function getIconGroups($dbconnection, $id=NULL)
	{
		$sql = "SELECT group_id, group_name
				FROM plan_icon_groups";

		$params = array();
		if(!empty($id) && is_numeric($id))
		{
			$sql .= " WHERE group_id = :id";
			$params[':id'] = $id;
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$groups = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$groups[] = array(
				'id' => $row->group_id,
				'name' => $row->group_name
			);
		}

		return $groups;
	}
	/**
	*** Récupère les groupes d'icones ainsi que les icones qu'ils contiennent
	*** @param PDO $dbconnection un objet PDO
	*** @param array $lang variables de langues du common.php
	*** @param integer (optionnel) $groupId l'identifiant du groupe d'icône si on ne veut en récupéré qu'un
	*** @return array liste de groups d'icones contenant une liste d'icones d'objets
	*** @throws \EsiaDbException
	**/
	public static function getIconsByGroups($dbconnection, $lang, $groupId=NULL)
	{
		$sql = "SELECT icon_id, icon_name, icon_path, icon_type, icon_group_id, group_name
				FROM plan_object_icons
				LEFT JOIN plan_icon_groups ON icon_group_id = group_id";

		$params = array();
		if(!empty($groupId) && is_numeric($groupId))
		{
			$sql .= " WHERE group_id = :group_id";
			$params[':group_id'] = $groupId;
		}

		$sql .= " ORDER BY icon_name";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$list = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!empty($row->group_name))
			{
				$groupName = $row->group_name;
				$groupId = $row->icon_group_id;
			}
			else
			{
				$groupName = $lang['PLAN_ICONS_DEFAULT_GROUP'];
				$groupId = 0;
			}

			$list[$groupName][] = array(
				'id' => $row->icon_id,
				'name' => $row->icon_name,
				'path' => $row->icon_path,
				'type' => $row->icon_type,
				'group' => $groupId
			);
		}

		return $list;
	}
	/**
	*** Ajouter une icone d'objet
	*** @param PDO $dbconnection un objet PDO
	*** @param string $name le nom de l'icone
	*** @param string $path le chemin vers le fichier image
	*** @param string $type extension du fichier image
	*** @param integer (optionnel) $groupId l'identifiant du groupe d'icones dans lequel ajouter cette icone
	*** @return integer id de la nouvelle icone en DB
	*** @throws \EsiaDbException
	**/
	public static function addIcon($dbconnection, $name, $path, $type, $groupId=NULL)
	{
		if(empty($name) || empty($path) || empty($type))
		{
			throw new \EsiaDbException("Missing icon required data");
		}

		$params = array(
			':name' => $name,
			':icn_path' => $path,
			':type' => $type
		);

		if(empty($groupId) || !is_numeric($groupId))
		{
			$sql = "INSERT INTO plan_object_icons (icon_id, icon_name, icon_path, icon_type, icon_group_id)
					VALUES(DEFAULT, :name, :icn_path, :type, NULL)
					RETURNING icon_id";
		}
		else
		{
			$sql = "INSERT INTO plan_object_icons (icon_id, icon_name, icon_path, icon_type, icon_group_id)
					VALUES(DEFAULT, :name, :icn_path, :type, :group_id)
					RETURNING icon_id";

			$params[':group_id'] = $groupId;
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$iconId = $query->fetch(PDO::FETCH_OBJ)->icon_id;

		return $iconId;
	}
	/**
	*** Ajouter un groupe d'icônes
	*** @param PDO $dbconnection un objet PDO
	*** @param string $name le nom du groupe d'icones
	*** @return integer id du nouveau groupe d'icones en DB
	*** @throws \EsiaDbException
	**/
	public static function addIconGroup($dbconnection, $name)
	{
		if(empty($name))
		{
			throw new \EsiaDbException("Missing icon group name");
		}

		$sql = "INSERT INTO plan_icon_groups (group_id, group_name)
				VALUES(DEFAULT, :name)
				RETURNING group_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, array(':name' => $name));

		$groupId = $query->fetch(PDO::FETCH_OBJ)->group_id;

		return $groupId;
	}
	/**
	*** Modifier une icone d'objet
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id identifiant de l'icone
	*** @param string $name le nom de l'icone
	*** @param string $path le chemin vers le fichier image
	*** @param string $type extension du fichier image
	*** @param integer (optionnel) $groupId l'identifiant du groupe d'icones dans lequel mettre cette icone (0 peut être passer pour remplacer par NULL en DB)
	*** @throws \EsiaDbException
	**/
	public static function updateIcon($dbconnection, $id, $name, $path, $type, $groupId=NULL)
	{
		if(empty($id) || !is_numeric($id))
		{
			throw new \EsiaDbException("Icon ID is not numeric");
		}

		$sql = "UPDATE plan_object_icons SET icon_name = :name, icon_path = :icn_path, icon_type = :type";

		$params = array(
			':name' => $name,
			':icn_path' => $path,
			':type' => $type
		);

		if(is_numeric($groupId))
		{
			if($groupId == 0)
			{
				$sql .= ", icon_group_id = DEFAULT";
			}
			else
			{
				$sql .= ", icon_group_id = :group_id";
				$params[':group_id'] = $groupId;
			}
		}

		$sql .= " WHERE icon_id = :id
				RETURNING icon_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Icon does not exist');
		}
	}
	/**
	*** Modifier le groupe id d'une icone d'objet
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id identifiant de l'icone
	*** @param integer $groupId l'identifiant du groupe d'icones dans lequel mettre cette icone (0 peut être passer pour remplacer par NULL en DB)
	*** @throws \EsiaDbException
	**/
	public static function updateIconGroupId($dbconnection, $id, $groupId)
	{
		if(empty($id) || !is_numeric($id))
		{
			throw new \EsiaDbException("Icon ID is not numeric");
		}
		if(!is_numeric($groupId))
		{
			throw new \EsiaDbException("Icon group ID is not numeric");
		}

		$params = array(
			':id' => $id
		);

		if($groupId == 0)
		{
			$sql = "UPDATE plan_object_icons SET icon_group_id = DEFAULT";
		}
		else
		{
			$sql = "UPDATE plan_object_icons SET icon_group_id = :group_id";
			$params[':group_id'] = $groupId;
		}

		$sql .= " WHERE icon_id = :id
				RETURNING icon_id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Icon does not exist');
		}
	}
	/**
	*** Modifier un groupe d'icônes
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id identifiant du groupe d'icones
	*** @param string $name le nom du groupe d'icones
	*** @throws \EsiaDbException
	**/
	public static function updateIconGroup($dbconnection, $id, $name)
	{
		if(empty($id) || !is_numeric($id))
		{
			throw new \EsiaDbException("Icon group ID is not numeric");
		}

		$sql = "UPDATE plan_icon_groups SET group_name = :name
				WHERE group_id = :id
				RETURNING group_id";

		$params = array(
			':id' => $id,
			':name' => $name
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Icon group does not exist');
		}
	}
	/**
	*** Supprime une icone d'objets
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id identifiant de l'icone
	*** @throws \EsiaDbException
	**/
	public static function deleteIcon($dbconnection, $id)
	{
		$params = array(':id' => $id);

		// Update plan objects which are using this icon by the default icon
		$sql = "UPDATE plan_objects SET object_icon_id = (
					SELECT icon_id FROM plan_object_icons WHERE icon_name='default'
				) WHERE object_icon_id = :id";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		// Delete icon
		$query = $dbconnection->prepare("DELETE FROM plan_object_icons WHERE icon_id = :id RETURNING icon_id");
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Icon does not exist');
		}
	}
	/**
	*** Supprime toutes les icone d'objets d'un groupe d'îcones
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $groupId identifiant du groupe d'îcones pour lequel on supprime toutes ses îcones
	*** @throws \EsiaDbException
	**/
	public static function deleteIconsByGroup($dbconnection, $groupId)
	{
		if(empty($groupId))
		{
			throw new \EsiaDbException('Group id is required to delete its icons');
		}

		$params = array(':id' => $groupId);

		// Update plan objects which are using icons from this icon group by the default icon
		$sql = "UPDATE plan_objects SET object_icon_id = (
					SELECT icon_id FROM plan_object_icons WHERE icon_name='default'
				) WHERE object_icon_id IN (
					SELECT icon_id FROM plan_object_icons WHERE icon_group_id = :id
				)";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		// Delete all icons from the group
		$query = $dbconnection->prepare("DELETE FROM plan_object_icons WHERE icon_group_id = :id");
		$query = Model_esia::executeStatement($query, $params);
	}
	/**
	*** Supprime un groupe d'icônes
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $id identifiant du groupe d'icones
	*** @throws \EsiaDbException
	**/
	public static function deleteIconGroup($dbconnection, $id)
	{
		$query = $dbconnection->prepare("UPDATE plan_object_icons SET icon_group_id = DEFAULT WHERE icon_group_id = :id");
		$query = Model_esia::executeStatement($query, array(':id' => $id));

		$query = $dbconnection->prepare("DELETE FROM plan_icon_groups WHERE group_id = :id RETURNING group_id");
		$query = Model_esia::executeStatement($query, array(':id' => $id));

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Icon group does not exist');
		}
	}
	/**
	*** Récupère les types de plans
	*** @param PDO $dbconnection un objet PDO
	*** @return array tous les types de plans disponnibles (id, name, controller)
	*** @param integer (optionnel) $typeId identifiant du type de plans pour lequel on veut récupérer les infos. Si NULL renvoi tous les types
	*** @param integer (optionnel) $planId identifiant du plan dont on veut récupérer le type
	*** @throws \EsiaDbException
	**/
	public static function getPlanTypes($dbconnection, $typeId=NULL, $planId=NULL)
	{
		$sql = "SELECT type_id, type_name, type_controller
				FROM plan_types";

		$params = array();
		if(!empty($typeId) && is_numeric($typeId))
		{
			$sql .= " WHERE type_id = :type_id";
			$params[':type_id'] = $typeId;
		}
		elseif(!empty($planId) && is_numeric($planId))
		{
			$sql .= " WHERE type_id = (SELECT plan_type_id FROM plan_plans WHERE plan_id = :plan_id)";
			$params[':plan_id'] = $planId;
		}

		$sql .= " ORDER BY type_name ASC";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$types = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$types[] = array(
				'id' => $row->type_id,
				'name' => $row->type_name,
				'controller' => $row->type_controller
			);
		}

		return $types;
	}
	/**
	*** Converti un ou plusieurs id en array d'ids depuis n'importe quel type de variable
	*** @param string|integer|array $ids le ou les id(s) à traiter
	*** @return array une liste d'ids
	**/
	private function convertIdsIntoArray($ids)
	{
		$list = array();
		if(is_array($ids))
		{
			foreach($ids as $id)
			{
				if(is_numeric($id))
				{
					$list[] = intval($id);
				}
			}
		}
		else
		{
			$list = array_map('intval', explode(",", $ids));
		}

		return $list;
	}
	/**
	*** Retourne l'état d'un objet plan passé en paramètre
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur
	*** @param array $objParams un tableau associatif des paramètres de l'objet (JSON en DB)
	*** @param string (optionnel) $objType nom du type de l'objet. Peut être laisser vide dans le cas ou l'on fournis nous meme le $objParams formaté comme l'attends cette fonction
	*** @param integer (optionnel) $objLinkPlan identifiant du plan lié a l'objet. Ou NULL si pas de plan lié
	*** @param integer $lvl (default: 0) niveau de récursivité actuelle
	*** @param integer $maxLvl (default: 4) nombre maximum d'appels de fonction autorisé en récusif
	*** @return string un champ texte avec le statut de l'objet (ok, warning, critical, unknown ou pending)
	*** @throws \EsiaDbException
	**/
	public function getObjectState($dbconnection, $userId, $objParams, $objType="", $objLinkPlan=NULL, $lvl=0, $maxLvl=Model_plan::PLAN_OBJ_STATE_MAX_LVL)
	{
		$globalState = 'ok';
		if($lvl < $maxLvl || $objType == "service" || $objType == "text")
		{
			// Si le parametre getstate n'est pas préciser et tant qu'il ne vaut pas "no" on récupere le status de l'objet
			if(!array_key_exists('getstate', $objParams) || $objParams['getstate'] != 'no')
			{
				// Etat des noeuds
				$nodes = array();
				if($objType == "rack_item" && !empty($objParams['config']['device_id']))
				{
					$nodes[] = intval($objParams['config']['device_id']);
				}
				elseif(!empty($objParams['nodes']))
				{
					$nodes = Model_plan::convertIdsIntoArray($objParams['nodes']);
				}

				if(!empty($nodes))
				{
					$listState = Model_esia::countNodeState($dbconnection, $nodes);
					$globalState = Model_plan::getMostImportantState($globalState, $listState);
					if($globalState == 'critical')
					{
						return 'critical';
					}
				}

				// Etat des groupes de noeuds
				$groups = array();
				if(!empty($objParams['groups']))
				{
					$groups = Model_plan::convertIdsIntoArray($objParams['groups']);
				}

				foreach($groups as $groupId)
				{
					$stateParent =  Model_esia::getGroupNodeStates($dbconnection, $userId, $groupId, FALSE);
					$globalState = Model_plan::getMostImportantState($globalState, $stateParent[$groupId]);
					if($globalState == 'critical')
					{
						return 'critical';
					}
					else
					{
						$stateChild = Model_esia::getGroupNodeStates($dbconnection, $userId, $groupId, TRUE);
						foreach($stateChild as $current)
						{
							$globalState = Model_plan::getMostImportantState($globalState, $current);
							if($globalState == 'critical')
							{
								return 'critical';
							}
						}
					}
				}

				// Etat des services
				$services = array();
				if(!empty($objParams['service']))
				{
					$services = Model_plan::convertIdsIntoArray($objParams['service']);
				}
				elseif(!empty($objParams['services']))
				{
					$services = Model_plan::convertIdsIntoArray($objParams['services']);
				}

				foreach($services as $serviceId)
				{
					$service = Model_esia::getServices($dbconnection, NULL, $serviceId)[0];
					$globalState = Model_plan::getMostImportantState($globalState, intval($service['state']));
					if($globalState == 'critical')
					{
						return 'critical';
					}
				}

				// Etat des plans
				if(!empty($objLinkPlan) && is_numeric($objLinkPlan))
				{
					$statePlan = Model_plan::getObjectPlanLinkedState($dbconnection, $userId, $objLinkPlan, $lvl+1, $maxLvl);
					$globalState = Model_plan::getMostImportantState($globalState, $statePlan);
				}
			}
		}

		return $globalState;
	}
	/**
	*** Retourne l'état global d'un plan donnée
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $userId identifiant de l'utilisateur
	*** @param integer $id identifiant du plan
	*** @param integer $lvl (default: 0) niveau de récursivité actuelle
	*** @param integer $maxLvl (default: 4) nombre maximum d'appels de fonction autorisé en récusif
	*** @return string un champ texte avec le statut du plan (ok, warning, critical, unknown ou pending)
	*** @throws \EsiaDbException
	**/
	public function getObjectPlanLinkedState($dbconnection, $userId, $id, $lvl=0, $maxLvl=Model_plan::PLAN_OBJ_STATE_MAX_LVL)
	{
		$globalState = 'ok';
		if($lvl < $maxLvl)
		{
			$sql = "SELECT object_link_plan_id, object_params, type_name
					FROM plan_objects
					LEFT JOIN plan_object_types ON type_id = object_type_id
					WHERE object_on_plan_id = :plan_id";

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query, array(':plan_id' => $id));

			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$objParams = json_decode($row->object_params, TRUE);

				// Ne pas récupérer les status des objets text et services au travers d'un plan lié
				if($row->type_name != 'text' && $row->type_name != 'service')
				{
					$stateObj = Model_plan::getObjectState($dbconnection, $userId, $objParams, $row->type_name, $row->object_link_plan_id, $lvl, $maxLvl);
					$globalState = Model_plan::getMostImportantState($globalState, $stateObj);
				}
			}
		}

		return $globalState;
	}
	/**
	*** Fait une comparaison entre 2 états pour retourner le plus prioritaire
	*** @param string $globalState le status actuelle sous forme de chaine de caractère (ok, warning, critical, unknown ou pending)
	*** @param array $states l'état que l'on souhaite tester sous forme de tableau associatif(voir getGroupNodeStates) ou sous forme de chaine de caractère ou d'integer
	*** @return string un champ texte avec le statut le plus critique (critical, warning, unknown, ok)
	**/
	public function getMostImportantState($globalState, $states)
	{
		$listStates = array(
			'critical' => 0,
			'warning' => 0,
			'unknown' => 0,
			'ok' => 0
		);

		if(is_numeric($states))
		{
			switch($states)
			{
				case 1:
					$listStates['warning']++;
					break;
				case 2:
					$listStates['critical']++;
					break;
				case 3:
					$listStates['unknown']++;
					break;
				default:
					$listStates['ok']++;
					break;
			}
		}
		elseif(is_string($states))
		{
			$listStates[strtolower($states)]++;
		}
		elseif(is_array($states))
		{
			// S'assurer que toutes les clées du tableau soient en minuscule
			$listStates = array_change_key_case($states, CASE_LOWER);
		}

		// Ajouter globalState dans la liste
		$listStates[$globalState]++;

		// Retourner l'état le plus prioritaire
		foreach(Model_plan::PLAN_PRIORITY_STATES as $p)
		{
			if($listStates[$p] > 0)
			{
				return $p;
			}
		}

		return $globalState; // Par securité si on ne match pas globalState dans les PLAN_PRIORITY_STATES on retourne la valeure envoyée dans globalState
	}
	/**
	*** Retourne un liste d'erreur survenue sur un objet plan
	*** @param PDO $dbconnection un objet PDO
	*** @param OBJECT $user object User class de l'utilisateur courant
	*** @param array $lang variables de langues du common.php
	*** @param array $objParams un tableau associatif des paramètres de l'objet (JSON en DB)
	*** @param string $objType nom du type de l'objet
	*** @param integer (optionnel) $objLinkPlan identifiant du plan lié a l'objet. Ou NULL si pas de plan lié
	*** @param array $alerts alertes préccédentes retourées par cette fonction si utilisée récursivement
	*** @param integer $lvl (default: 0) niveau de récursivité actuelle
	*** @param integer $maxLvl (default: 4) nombre maximum d'appels de fonction autorisé en récusif
	*** @return array liste d'alertes survenue sur cet objet ou plan (name, service, message, time)
	*** @throws \EsiaDbException
	**/
	public function getObjectAlerts($dbconnection, $user, $lang, $objParams, $objType, $objLinkPlan=NULL, $alerts=array(), $lvl=0, $maxLvl=Model_plan::PLAN_OBJ_ALERTS_MAX_LVL)
    {
		if($lvl < $maxLvl)
		{
			// Si le parametre getstate n'est pas préciser et tant qu'il ne vaut pas "no" on récupere le status de l'objet
			if(!array_key_exists('getstate', $objParams) || $objParams['getstate'] != 'no')
			{
				// Alertes sur les noeuds
				$nodes = array();
				if($objType == "rack_item" && !empty($objParams['config']['device_id']))
				{
					$nodes[] = intval($objParams['config']['device_id']);
				}
				elseif(!empty($objParams['nodes']))
				{
					$nodes = Model_plan::convertIdsIntoArray($objParams['nodes']);
				}

				foreach($nodes as $nodeId)
				{
					// Comme cette fonction est récursive on vérifie que l'on a pas déjà récupérer les données de ce noeud sur un autre objet
					if(!isset($alerts['nodes'][$nodeId]))
					{
						$node = Model_esia::InstanceNode($dbconnection, $nodeId);

						if($user->getAccess($dbconnection, "device_view", $node->getName()) == 1)
						{
							// On récupère les services en erreur sur le noeud
							$nodeState = 'ok';
							$listErrors = array();
							$errors = Model_esia::getNodeError($dbconnection, $nodeId);
							foreach($errors as $error)
							{
								if(in_array($error['type'], Model_plan::PLAN_STATES_IGNORED))
								{
									continue;
								}
								$commonName = Model_esia::getCommonName($lang, $error['service'], $error['plugin_name']);

								$listErrors[] = array(
									'type' => $error['type'],
									'service' => $commonName,
									'message' => $error['message'],
									'time' => $error['time']
								);

								$nodeState = Model_plan::getMostImportantState($nodeState, $error['type']);
							}

							// Récupérer l'url du noeud
							$pageControl = Model_esia::getPageControl($dbconnection, $node->getType());
							$pageControl = Model_esia::replacePageControlVariable($pageControl, $nodeId);

							$alerts['nodes'][$nodeId] = array(
								'id' => $nodeId,
								'name' => $node->getName(),
								'page_control' => $pageControl . $nodeId,
								'state' => $nodeState,
								'errors' => $listErrors
							);
						}
					}
				}

				// Alertes sur les groupes
				$groups = array();
				if(!empty($objParams['groups']))
				{
					$groups = Model_plan::convertIdsIntoArray($objParams['groups']);
				}

				foreach($groups as $groupId)
				{
					// Comme cette fonction est récursive on vérifie que l'on a pas déjà récupérer les données de ce groupe de noeuds sur un autre objet
					if(!isset($alerts['groups'][$groupId]))
					{
						$groupState = 'ok';

						$stateParent =  Model_esia::getGroupNodeStates($dbconnection, $user->getId(), $groupId, FALSE);
						$groupState = Model_plan::getMostImportantState($groupState, $stateParent[$groupId]);
						if($groupState != 'critical')
						{
							$stateChild = Model_esia::getGroupNodeStates($dbconnection, $user->getId(), $groupId, TRUE);
							foreach($stateChild as $current)
							{
								$groupState = Model_plan::getMostImportantState($groupState, $current);
								if($groupState == 'critical')
								{
									break;
								}
							}
						}

						$groupInfos = Model_esia::getGroupDeviceIn($dbconnection, $groupId)[0];

						$alerts['groups'][$groupId] = array(
							'id' => $groupId,
							'name' => $groupInfos['name'],
							'page_control' => "/node_index.php?nodegroup=" . $groupId,
							'state' => $groupState
						);
					}
				}

				// Alertes sur les services
				$services = array();
				if($objType == "service")
				{
					foreach($objParams['services'] as $srv)
					{
						$services[] = intval($srv['id']);
					}
				}
				elseif(!empty($objParams['services']))
				{
					$services = Model_plan::convertIdsIntoArray($objParams['services']);
				}

				foreach($services as $serviceId)
				{
					// Comme cette fonction est récursive on vérifie que l'on a pas déjà récupérer les données de ce noeud sur un autre objet
					if(!isset($alerts['services'][$serviceId]))
					{
						$service = Model_esia::getServices($dbconnection, NULL, $serviceId)[0];
						// Utiliser getMostImportantState pour récupérer le status en string
						$serviceState = Model_plan::getMostImportantState('ok', $service['state']);

						if($user->getAccess($dbconnection, "device_view", $service['device_name']) == 1)
						{
							// Récupérer des données a propos de l'erreur s'il y en a
							$error = array('message' => '', 'time' => '');
							if($serviceState != 'ok')
							{
								$error = Model_esia::getNodeError($dbconnection, $service['device_id'], $serviceId)[0];
							}

							$commonName = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);

							// Récupérer l'url du noeud
							$pageControl = Model_esia::getPageControl($dbconnection, $service['device_type']);
							$pageControl = Model_esia::replacePageControlVariable($pageControl, $service['device_id']);

							$alerts['services'][$serviceId] = array(
								'id' => $serviceId,
								'device_name' => $service['device_name'],
								'page_control' => $pageControl . $service['device_id'],
								'service' => $commonName,
								'message' => $error['message'],
								'time' => $error['time'],
								'state' => $serviceState
							);
						}
					}
				}

				// Alertes sur les plans
				if(!empty($objLinkPlan) && is_numeric($objLinkPlan))
				{
					$alerts = Model_plan::getObjectPlanLinkedAlerts($dbconnection, $user, $lang, $objLinkPlan, $alerts, $lvl+1, $maxLvl);
				}
			}
		}

		return $alerts;
    }
	/**
	*** Retourne un liste d'erreur survenue sur un plan donnée
	*** @param PDO $dbconnection un objet PDO
	*** @param OBJECT $user object User class de l'utilisateur courant
	*** @param array $lang variables de langues du common.php
	*** @param integer $id identifiant du plan
	*** @param array $alerts alertes préccédentes retourées par cette fonction si utilisée récursivement
	*** @param integer $lvl (default: 0) niveau de récursivité actuelle
	*** @param integer $maxLvl (default: 4) nombre maximum d'appels de fonction autorisé en récusif
	*** @return array liste d'alertes survenue sur cet objet ou plan (name, service, message, time)
	*** @throws \EsiaDbException
	**/
	public function getObjectPlanLinkedAlerts($dbconnection, $user, $lang, $id, $alerts=array(), $lvl=0, $maxLvl=Model_plan::PLAN_OBJ_ALERTS_MAX_LVL)
	{
		if($lvl < $maxLvl)
		{
			$sql = "SELECT object_link_plan_id, object_params, type_name
					FROM plan_objects
					LEFT JOIN plan_object_types ON type_id = object_type_id
					WHERE object_on_plan_id = :plan_id";

			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query, array(':plan_id' => $id));

			while($row = $query->fetch(PDO::FETCH_OBJ))
			{
				$objParams = json_decode($row->object_params, TRUE);

				// Ne pas récupérer les alertes des objets text et services au travers d'un plan lié
				if($row->type_name != 'text' && $row->type_name != 'service')
				{
					$alerts = Model_plan::getObjectAlerts($dbconnection, $user, $lang, $objParams, $row->type_name, $row->object_link_plan_id, $alerts, $lvl, $maxLvl);
				}
			}
		}

		return $alerts;
	}
	/**
	*** Récupère une partie des paramètres d'un plan en spécifiant le chemin de l'objet JSON
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId identifiant du plan
	*** @param array $jsonPath chemin de l'objet JSON vers la valeur recherchée. Ex:"values":[{"id":"117","name":"Total","order":0}] $jsonPath=array("values", "0", "id") retourne 117
	*** @return array object le parametre demander
	*** @throws \EsiaDbException
	**/
	public static function getPlanParamJSON($dbconnection, $planId, $jsonPath=array())
	{
		$sql = "SELECT plan_params FROM plan_plans WHERE plan_id=:plan_id";
		$params = array(':plan_id' => $planId);

		if(is_array($jsonPath) && !empty($jsonPath))
		{
			$sql = "SELECT json_extract_path(plan_params::json";

			// Set JSON path to the requested value
			foreach ($jsonPath as $key => $value)
			{
				$sql .= ", :" . $key;
				$params[':' . $key] = $value; // Check every values with PDO
			}

			// Don't forget the json_extract_path function closure parenthesis
			$sql .= ") FROM plan_plans WHERE plan_id=:plan_id";
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$json = $query->fetch(PDO::FETCH_COLUMN, 0);

		return json_decode($json, TRUE);
	}
	/**
	*** Récupère une partie des paramètres d'un object du plan en spécifiant le chemin de l'objet JSON
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $objectId identifiant de l'objet
	*** @param array $jsonPath chemin de l'objet JSON vers la valeur recherchée. Ex:"values":[{"id":"117","name":"Total","order":0}] $jsonPath=array("values", "0", "id") retourne 117
	*** @return array object le parametre demander
	*** @throws \EsiaDbException
	**/
	public static function getObjectParamJSON($dbconnection, $objectId, $jsonPath=array())
	{
		$sql = "SELECT object_params FROM plan_objects WHERE object_id=:object_id";
		$params = array(':object_id' => $objectId);

		if(is_array($jsonPath) && !empty($jsonPath))
		{
			$sql = "SELECT json_extract_path(object_params::json";

			// Set JSON path to the requested value
			foreach ($jsonPath as $key => $value)
			{
				$sql .= ", :" . $key;
				$params[':' . $key] = $value; // Check every values with PDO
			}

			// Don't forget the json_extract_path function closure parenthesis
			$sql .= ") FROM plan_objects WHERE object_id=:object_id";
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$json = $query->fetch(PDO::FETCH_COLUMN, 0);

		return json_decode($json, TRUE);
	}
	/**
	*** Update une partie des paramètres d'un plan en spécifiant le chemin de l'objet JSON. Tout le contenu à partir du path sera remplacer par le nouveau. Si le path n'existe pas rien ne sera modifier excepter si c'est uniquement la dernière clée du path, celle-ci sera alors créée.
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId identifiant du plan
	*** @param json $json nouvelle valeur des paramètres en JSON
	*** @param array $jsonPath chemin de l'objet JSON vers la valeur recherchée. Ex:"values":[{"id":"117","name":"Total","order":0}] $jsonPath=array("values", "0", "id")
	*** @throws \EsiaDbException
	**/
	public static function updatePlanParamJSON($dbconnection, $planId, $json, $jsonPath=array())
	{
		$sql = "UPDATE plan_plans
				SET plan_params=:json_param::json
				WHERE plan_id=:plan_id";

		$params = array(
			':plan_id' => $planId,
			':json_param' => $json
		);

		if(is_array($jsonPath) && !empty($jsonPath))
		{
			$jsonPath = '"' . implode('","', $jsonPath) . '"';

			$sql = "UPDATE plan_plans
					SET plan_params = jsonb_set(plan_params::jsonb, '{" . $jsonPath . "}', :json_param)::json
					WHERE plan_id=:plan_id";
		}

		$query = $dbconnection->prepare($sql);
		Model_esia::executeStatement($query, $params);
	}
	/**
	*** Update une partie des paramètres d'un object du plan en spécifiant le chemin de l'objet JSON. Tout le contenu à partir du path sera remplacer par le nouveau. Si le path n'existe pas rien ne sera modifier excepter si c'est uniquement la dernière clée du path, celle-ci sera alors créée.
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $objectId identifiant de l'objet
	*** @param json $json nouvelle valeur des paramètres en JSON
	*** @param array $jsonPath chemin de l'objet JSON vers la valeur à modifier. Ex:"values":[{"id":"117","name":"Total","order":0}] $jsonPath=array("values", "0", "name")
	*** @throws \EsiaDbException
	**/
	public static function updateObjectParamJSON($dbconnection, $objectId, $json, $jsonPath=array())
	{
		$sql = "UPDATE plan_objects
				SET object_params=:json_param::json
				WHERE object_id=:object_id";

		$params = array(
			':object_id' => $objectId,
			':json_param' => $json
		);

		if(is_array($jsonPath) && !empty($jsonPath))
		{
			$jsonPath = '"' . implode('","', $jsonPath) . '"';

			$sql = "UPDATE plan_objects
					SET object_params = jsonb_set(object_params::jsonb, '{" . $jsonPath . "}', :json_param)::json
					WHERE object_id=:object_id";
		}

		$query = $dbconnection->prepare($sql);
		Model_esia::executeStatement($query, $params);
	}
	/**
	*** Récupère le nom d'un noeud via son ID. Fonction simplifiée a n'utilisé que s'il n'est pas nécessaire de vérifier que l'user à les accès au noeud en question. Sinon utiliser les fonction du Model_esia
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $objectId identifiant du noeud
	*** @return string le nom du noeud
	*** @throws \EsiaDbException
	**/
	public static function getDeviceNameById($dbconnection, $nodeId)
	{
		$sql = "SELECT device_name
				FROM esia_device
				WHERE device_id=:device_id";

		$params = array(':device_id' => $nodeId);

		$query = $dbconnection->prepare($sql);
		Model_esia::executeStatement($query, $params);

		return $query->fetch(PDO::FETCH_COLUMN, 0);
	}
	/**
	*** Récupère les objets qui sont liés à un plan. Nom de l'objet et nom du plan auquel appartient l'objet
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId identifiant du plan vers lequel les objets pointent
	*** @return array liste des objets. object_name => nom de l'objet, plan_name => nom du plan auquel appartient l'objet
	*** @throws \EsiaDbException
	**/
	public static function getObjectsLinkedOnPlan($dbconnection, $planId)
	{
		$sql = "SELECT object_name, plan_name
				FROM plan_objects
				LEFT JOIN plan_plans ON plan_id = object_on_plan_id
				WHERE object_link_plan_id = :plan_id
				ORDER BY plan_name, object_name";

		$params = array(':plan_id' => $planId);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$objects = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$objects[] = array(
				'plan_name' => $row->plan_name,
				'object_name' => $row->object_name
			);
		}

		return $objects;
	}
	/**
	*** Récupère les services d'un noeud par un ou plusieurs plugin_tag
	*** @param PDO $dbconnection un objet PDO
	*** @param string|array $tags liste de tags a rechercher sous forme d'array ou de string dont les éléments sont séparer par des ,
	*** @param integer (optionnel) $deviceId identifiant du noeud sur lequel filtrer les services. Si vide pâs de filtre sur le noeud
	*** @return integer identifiant plan_id du dossier
	*** @throws \EsiaDbException
	**/
	public static function getServicesByPluginTags($dbconnection, $tags, $deviceId=NULL)
	{
		if(is_string($tags))
		{
			$tags = explode(',', $tags);
		}

		if(empty($tags) || !is_array($tags))
		{
			throw new \EsiaDbException('Plugin tags are required');
		}

		$regex = implode('|', $tags);

		$sql = "SELECT service_id, service_name, service_plugin_id, service_device_id, service_previous_state, service_last_stdout
				FROM esia_services
				WHERE service_plugin_id IN (
					SELECT plugin_id FROM esia_plugins WHERE plugin_tag ~* '" . $regex . "'
				)";

		$params = array();

		if(!empty($deviceId) && is_numeric($deviceId))
		{
			$sql .= " AND service_device_id = :device_id";
			$params[':device_id'] = $deviceId;
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		$services = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$services[$row->service_id] = array(
				'id' => $row->service_id,
				'name' => $row->service_name,
				'plugin_id' => $row->service_plugin_id,
				'device_id' => $row->service_device_id,
				'real_state' => $row->service_previous_state,
				'stdout' => $row->service_last_stdout
			);
		}

		return $services;
	}
	/**
	*** Récupère le nom d'un object du plan via son ID
	*** @param PDO $dbconnection un objet PDO
	*** @param integer $planId id du plan auquel appartienent ces objets
	*** @param array||integer $idsList liste d'identifiants d'objets plan
	*** @return string le nom du noeud
	*** @throws \EsiaDbException
	**/
	public static function getObjectsName($dbconnection, $planId, $idsList=array())
	{
		$sql = "SELECT object_id, object_name
				FROM plan_objects
				WHERE object_on_plan_id = :plan_id";

		$params = array(':plan_id' => $planId);

		if(!empty($idsList))
		{
			if(is_numeric($idsList))
			{
				$sql .= " AND object_id = :object_id";
				$params[':object_id'] = $idsList;
			}
			elseif(is_array($idsList))
			{
				foreach($idsList as $id)
				{
					if(!is_numeric($id))
					{
						throw new \EsiaDbException('Objects list IDs must be numeric');
					}
				}
				$idsStr = implode(',', $idsList);
				$sql .= " AND object_id IN (" . $idsStr . ")";
			}
			else
			{
				throw new \EsiaDbException('Object ID must be numeric');
			}
		}

		$query = $dbconnection->prepare($sql);
		Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new \EsiaDbException('Object does not exist on this plan: ' . $planId);
		}

		$objects = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$objects[$row->object_id] = $row->object_name;
		}

		return $objects;
	}
}
?>
