/*  @name: plan_folder.js
**
**  @description: JavaScript utility functions for plan folders pages
**  @author: Quentin George <q.george@esia-sa.com>
**  @date: 30/03/2022
*/


//Vérifie si le menu popup d'ajout d'un plan ne dépasse pas de la fenètre. Se base sur la position du bouton toggler car contrairement a celles du formulaire elles ne varient pas.
function setAddPopUpSafePos(toggler, popup, container) {
	if ((toggler.offset().left - 60) < (container.offset().left + 10)) {
		//A gauche (-60 = initial popup pos left, +10 = container padding left)
		popup.css("left", "0px");
	} else if ((toggler.offset().left + popup.width() - 60) > (container.offset().left + container.width() - 20)) {
		//A droite (-20 = padding left & right)
		popup.css({
			"left": "auto",
			"right": "0px"
		});
	} else {
		//Réinitialise les valeurs
		popup.css({
			"left": "",
			"right": ""
		});
	}

	if ((toggler.offset().top + toggler.height() + popup.height()) > (container.offset().top + container.height() - 20)) {
		//En bas (-20 = padding top & bottom)
		popup.css({
			"top": "auto",
			"bottom": "65px"
		});
	} else {
		//Réinitialise les valeurs
		popup.css({
			"top": "",
			"bottom": ""
		});
	}
}

// Set les éléments déplacables
function setItemDraggable(item) {
	item.draggable({
		revert: "invalid",
		containment: ".plan-folder-background",
		helper: "clone",
		cursor: "move",
		start: function() {
			$(this).addClass("dragging-source");
		},
		stop: function() {
			$(this).removeClass("dragging-source");
		}
	});
}

// Set les éléments de destination
function setItemDroppable(item) {
	item.droppable({
		accept: ".plan-elem",
		hoverClass: "droppable-hover",
		activeClass: "droppable-active",
		drop: function(event, ui) {
			var item = $(ui.draggable),
				id = item.attr("data-id"),
				name = item.find(".plan-name").text(),
				parentId = $(this).attr("data-id"),
				type = item.attr("data-type");

			$.ajax({
				type: "POST",
				url: glob_apache_path + "folder_plan.php",
				data: {action: "move", id: id, name: name, parent: parentId, type: type},
				dataType: "json"
			})
			.done(function(data) {
				if (data.error) {
					// Error Message
					displayUserMessage(data.error, true);
				} else if (data.success) {
					displayUserMessage(data.success, false);

					// Safely remove element on current folder
					item.remove();
				}
			});
		}
	});
}

// Drag & Drop pour le déplacement des plans et dossiers
function planDragDrop() {
	// éléments déplacables
	setItemDraggable($(".plan-folder, .plan-file"));
	// éléments de destination
	setItemDroppable($(".plan-folder"));
}

// Get a new plan or folder position by alphabetical order. Return the attr data-id of the next element. In order to insert the new one before.
function getNewItemPosInList(items, newName) {
	newName = newName.toLowerCase();
	items.each(function(index, item) {
		var itemName = $(item).find(".plan-name").text().toLowerCase();

		if (itemName.localeCompare(newName) > 0) {
			nextItem = $(item);
			return false; // Break the each loop
		}
	});
}

// Cancel plan or folder add
function cancelNewItemCreation() {
	$(".plan-add-content").hide();
	$("#new_plan")[0].reset();
}

// Create new folder or plan with ajax
function createNewItem() {
	$("#new_plan").submit(function(e) {
		e.preventDefault();

		var folderId = $('#new_folder_id').val(),
			name = $('#new_plan_name').val(),
			type = $("input[name='plan_type']:checked").val();

		// Close create menu
		cancelNewItemCreation();

		$('html').css('cursor', 'progress');

		$.ajax({
			type: "POST",
			url: glob_apache_path + "folder_plan.php",
			data: {action: "create", folder_id: folderId, plan_name: name, plan_type: type},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true);
			} else if (data.success) {
				displayUserMessage(data.success, false);

				// Create the new item
				if(data.new) {
					var newItem = {};

					switch(data.new.type) {
						case 'folder':
							newItem = $('#default-items .plan-folder').clone();
							break;
						case 'rack':
							newItem = $('#default-items .default-plan-rack').clone();
							break;
						default:
							// image
							newItem = $('#default-items .default-plan-img').clone();
					}

					newItem.attr('data-id', data.new.id);
					newItem.find('.plan-name').text(data.new.name);
					newItem.find('a').attr({title: data.new.type_lang, href: data.new.url});
					newItem.hide(); // Hide first to fadeIn after append

					// Add the new item to the right place. In folders or plans by alphabetical order
					var items = [];
					if(data.new.type == "folder") {
						items = $('#folder-content .plan-folder');

						if(items.length == 0) {
							// If no other folders put the new one at the beginning of #folder-content
							$('#folder-content').prepend(newItem);
						}
					} else {
						items = $('#folder-content .plan-file');

						if(items.length == 0) {
							// If no other plans put the new one at the end of #folder-content but before the add button
							newItem.insertBefore($('#folder-content .plan-add'));
						}
					}

					if(items.length > 0) {
						// If other items for this type check the alphabetical position
						var newName = data.new.name.toLowerCase(),
							nextItem = 'END';

						items.each(function(index, item) {
							var itemName = $(item).find(".plan-name").text().toLowerCase();

							if (itemName.localeCompare(newName) > 0) {
								// Old item further in alphabetical order thant the new item
								nextItem = $(item);
								return false; // Break the each loop
							}
						});

						if (nextItem == 'END') {
							// If there is no item further than the new one it means that the new item should be placed at the latest position
							newItem.insertAfter($(items[items.length - 1]));
						} else {
							// Set the new item before the next item founded
							newItem.insertBefore(nextItem);
						}
					}

					newItem.fadeIn(600);

					// Set drag & drop
					setItemDraggable(newItem);
					if(data.new.type == "folder") {
						setItemDroppable(newItem);
					}
				}
			}

			$('html').css('cursor', '');
		})
		.fail(function() {
			$('html').css('cursor', '');
		});
	});
}

// Update folder with ajax
function updateFolder() {
	$("#edit_infos").submit(function(e) {
		e.preventDefault();

		$.ajax({
			type: "POST",
			url: glob_apache_path + "folder_plan.php",
			data: {action: "update", form_data: $(this).serialize()},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true);
			} else if (data.url) {
				// If no error reload the page to update breadcrumb, treeview and folder name
				window.location.reload();
			}
		});
	});
}

// Delete folder with ajax
function deleteFolder() {
	$("#delete_folder").submit(function(e) {
		e.preventDefault();

		var folderId = $('#delete_folder_id').val(),
			folderName = $('#delete_folder_name').val(),
			parent = $('#delete_folder_parent').val(),
			delContent = $("input[name='content_action']:checked").val();

		$.ajax({
			type: "POST",
			url: glob_apache_path + "folder_plan.php",
			data: {action: "delete", folder_id: folderId, folder_name: folderName, folder_parent: parent, content_action: delContent},
			dataType: "json"
		})
		.done(function(data) {
			if (data.error) {
				// Error Message
				displayUserMessage(data.error, true);
			} else if (data.url) {
				// If no error redirect the page
				window.location.replace(data.url);
			}
		});
	});
}
