<?php
/*
** @name: Plan.class.php
**
** @description: Main class of esia webp-plan
** @author: Quentin George <q.george@esia-sa.com>
** @date: 12/01/2022
*/
namespace Plan;

class Plan extends AbstractPlan
{
	protected $type_controller;
	protected $folder_id;
	protected $folder_name;
	protected $current_user_access;
	protected $list_access_users;
	protected $list_objects;
	/**
	***
	*** @NAME: __construct
	*** @DESC: Default Plan class constructor
	*** @IN: (Optional) (OBJECT) $plan => if it's a 'Plan' object, the new object will be a copy of this one
	*** @OUT: /
	*** @THROW: PlanException if the object is not a 'Plan' object
	**/
	public function __construct($plan=NULL)
	{
		if(is_object($plan))
		{
			if(get_class($plan) == 'Plan\Plan')
			{
				$this->constructClone($plan);
			}
			else
			{
				throw new PlanException('$plan is not a Plan object');
			}
		}
		else
		{
			$this->params = array();
			$this->state = 'unknown';
			$this->type_controller = '';
			$this->folder_name = '';
			$this->current_user_access = 'NONE';
			$this->list_access_users = array(
				'CREATOR' => array(),
				'WRITE' => array(),
				'READ' => array()
			);
			$this->list_objects = array();
		}
	}
	/**
	***
	*** @NAME: constructClone
	*** @DESC: Copy constructor
	*** @IN: (OBJECT) $plan => 'Plan' object
	*** @OUT: /
	*** @THROW: /
	**/
	public function constructClone(\Plan\Plan $plan)
	{
		$this->id = $plan->getId();
		$this->name = $plan->getName();
		$this->type_id = $plan->getTypeId();
		$this->type_name = $plan->getTypeName();
		$this->params = $plan->getParams();
		$this->state = $plan->getState();

		$this->type_controller = $plan->getTypeController();
		$this->folder_id = $plan->getFolderId();
		$this->folder_name = $plan->getFolderName();
		$this->current_user_access = $plan->getCurrentUserAccess();
		$this->list_access_users = $plan->getAccessUsersList();
		$this->list_objects = $plan->getListObjects();
	}
	/**
	***
	*** @NAME: toArray
	*** @DESC: return all class vars and methods results as an array
	*** @IN: (Optional) (BOOLEAN) $isObjects => If TRUE, return also all list objects class as array. If FALSE don't return objects
	*** @IN: (Optional) (STRING) $path => $_SERVER['DOCUMENT_ROOT'] If objects icon files are required (only if $isObjects is TRUE)
	*** @OUT: array all methods converted to array
	*** @THROW: /
	**/
	public function toArray($isObjects=FALSE, $path=NULL)
	{
		$arr = array(
			'id' => $this->id,
			'name' => $this->name,
			'type_id' => $this->type_id,
			'type_name' => $this->type_name,
			'params' => $this->params,
			'state' => $this->state,
			'state_code' => $this->getStateCode(),
			'type_controller' => $this->type_controller,
			'folder_id' => $this->folder_id,
			'folder_name' => $this->folder_name,
			'current_user_access' => $this->current_user_access,
			'list_access_users' => $this->list_access_users
		);

		if($isObjects)
		{
			$arr['list_objects'] = $this->getListObjects(TRUE, $path);
		}

		return $arr;
	}
	/**
	***
	*** @NAME: setTypeController
	*** @DESC: Set plan type controller path
	*** @IN: (STRING) $typeCtrl => typeCtrl of the plan
	*** @OUT: /
	*** @THROW: /
	**/
	public function setTypeController($typeCtrl)
	{
		$this->type_controller = $typeCtrl;
	}
	/**
	***
	*** @NAME: setFolderId
	*** @DESC: Set plan parent folder id
	*** @IN: (INTEGER) $id => id of the folder
	*** @OUT: /
	*** @THROW: /
	**/
	public function setFolderId($id)
	{
		$this->folder_id = $id;
	}
	/**
	***
	*** @NAME: setFolderName
	*** @DESC: Set plan parent folder name
	*** @IN: (STRING) $name => name of the folder
	*** @OUT: /
	*** @THROW: /
	**/
	public function setFolderName($name)
	{
		$this->folder_name = $name;
	}
	/**
	***
	*** @NAME: setCurrentUserAccess
	*** @DESC: Set the rights of the current user on this plan (only READ or WRITE regardless of CREATOR)
	*** @IN: (STRING) $access => access READ, WRITE or CREATOR
	*** @OUT: /
	*** @THROW: /
	**/
	public function setCurrentUserAccess($access)
	{
		switch($access)
		{
			case 'CREATOR':
			case 'WRITE':
				$this->current_user_access = 'WRITE';
				break;
			case 'READ':
				$this->current_user_access = 'READ';
				break;
			default:
				$this->current_user_access = 'NONE';
				break;
		}
	}
	/**
	***
	*** @NAME: setAccessUsersList
	*** @DESC: Set plan users privileges list
	*** @IN: (Optional) (ARRAY) $listRead => List of users id with READ privilege on this plan. Empty array for no users. NULL to not update this privilege
	*** @IN: (Optional) (ARRAY) $listWrite => List of users id with WRITE privilege on this plan. Empty array for no users. NULL to not update this privilege
	*** @IN: (Optional) (ARRAY) $listCreator => List of users id with CREATOR privilege on this plan. Empty array for no users. NULL to not update this privilege
	*** @OUT: /
	*** @THROW: /
	**/
	public function setAccessUsersList($listRead=NULL, $listWrite=NULL, $listCreator=NULL)
	{
		if(isset($listRead))
		{
			$this->list_access_users['READ'] = $listRead;
		}
		if(isset($listRead))
		{
			$this->list_access_users['WRITE'] = $listWrite;
		}
		if(isset($listRead))
		{
			$this->list_access_users['CREATOR'] = $listCreator;
		}
	}
	/**
	***
	*** @NAME: setAccessUser
	*** @DESC: Add or Update plan user privilege to the list
	*** @IN: (INTEGER) $userId => ID of the user
	*** @IN: (STRING) $privilege => Privilege of the user on this plan
	*** @OUT: /
	*** @THROW: /
	**/
	public function setAccessUser($userId, $privilege)
	{
		// If the user id is already in READ, WRITE or CREATOR array. Remove it before add it to the new one
		foreach($this->list_access_users as $priv => $idList)
		{
			if(in_array($idList, $userId))
			{
				unset($this->list_access_users[strtoupper($priv)][$userId]);
			}
		}

		$this->list_access_users[strtoupper($privilege)][$userId] = $userId;
	}
	/**
	***
	*** @NAME: deleteAccessUser
	*** @DESC: Remove plan object from list. Object must be 'PlanObject' class type
	*** @IN: (INTEGER) $userId => ID of the user to remove privilege
	*** @OUT: /
	*** @THROW: /
	**/
	public function deleteAccessUser($userId, $privilege)
	{
		unset($this->list_access_users[strtoupper($privilege)][$userId]);
	}
	/**
	***
	*** @NAME: setObject
	*** @DESC: Add or Update plan object to the list. Object must be 'PlanObject' class type
	*** @IN: (OBJECT) $object => 'PlanObject' class
	*** @OUT: /
	*** @THROW: /
	**/
	public function setObject($object)
	{
		$this->list_objects[$object->getId()] = new \Plan\ObjectPlan($object);
	}
	/**
	***
	*** @NAME: deleteObject
	*** @DESC: Remove plan object from list. Object must be 'PlanObject' class type
	*** @IN: (INTEGER) $objId => ID of the object to remove
	*** @OUT: /
	*** @THROW: /
	**/
	public function deleteObject($objId)
	{
		unset($this->list_objects[$objId]);
	}
	/**
	***
	*** @NAME: getTypeController
	*** @DESC: Return plan type controller path
	*** @IN: (Optional) (BOOLEAN) $isQuery => If TRUE return the full controller path with the query string. If FALSE return only the part of the controller whitout the query string.
	*** @OUT: Type controller of the plan
	*** @THROW: /
	**/
	public function getTypeController($isQuery=TRUE)
	{
		if($isQuery)
		{
			return $this->type_controller;
		}
		else
		{
			return explode('?', $this->type_controller)[0];
		}
	}
	/**
	***
	*** @NAME: getFolderId
	*** @DESC: Return plan parent folder id
	*** @IN: /
	*** @OUT: Plan folder id
	*** @THROW: /
	**/
	public function getFolderId()
	{
		return $this->folder_id;
	}
	/**
	***
	*** @NAME: getFolderName
	*** @DESC: Return plan parent folder name
	*** @IN: /
	*** @OUT: Plan folder name
	*** @THROW: /
	**/
	public function getFolderName()
	{
		return $this->folder_name;
	}
	/**
	***
	*** @NAME: getCurrentUserAccess
	*** @DESC: Return the rights (READ/WRITE) of the current user on this plan
	*** @IN: /
	*** @OUT: User rights on the plan
	*** @THROW: /
	**/
	public function getCurrentUserAccess()
	{
		return $this->current_user_access;
	}
	/**
	***
	*** @NAME: getAccessUsersList
	*** @DESC: Return plan users privileges list
	*** @IN: /
	*** @OUT: List of users privileges on this plan
	*** @THROW: /
	**/
	public function getAccessUsersList()
	{
		return $this->list_access_users;
	}
	/**
	***
	*** @NAME: getListObjects
	*** @DESC: Return plan objects list
	*** @IN: (Optional) (BOOLEAN) $isArray => If TRUE, return all class vars and methods results of the PlanObject class as an array
	*** @IN: (Optional) (STRING) $path => $_SERVER['DOCUMENT_ROOT'] If objects icon files are required (only if $isArray is TRUE)
	*** @OUT: Objects of the plan. 'PlanObject' class type
	*** @THROW: /
	**/
	public function getListObjects($isArray=FALSE, $path=NULL)
	{
		if($isArray)
		{
			$arr = array();
			foreach($this->list_objects as $objId => $object)
			{
				$arr[] = $object->toArray($path);
			}
			return $arr;
		}
		else
		{
			return $this->list_objects;
		}
	}
	/**
	***
	*** @NAME: getListObjectsByTypeName
	*** @DESC: Return plan objects list sorted by objects type name
	*** @IN: (Optional) (BOOLEAN) $isArray => If TRUE, return all class vars and methods results of the PlanObject class as an array
	*** @IN: (Optional) (STRING) $path => $_SERVER['DOCUMENT_ROOT'] If objects icon files are required (only if $isArray is TRUE)
	*** @OUT: Objects of the plan. 'PlanObject' class type sorted by object type name
	*** @THROW: /
	**/
	public function getListObjectsByTypeName($isArray=FALSE, $path=NULL)
	{
		$ret = array();
		if($isArray)
		{
			foreach($this->list_objects as $objId => $object)
			{
				$objArr = $object->toArray($path);
				$ret[$objArr['type_name']][] = $objArr; // No push by objectId as key to let json_encode create as JS array instead of JS object
			}
		}
		else
		{
			foreach($this->list_objects as $objId => $object)
			{
				$ret[$object->$obj->getTypeName()][$objId] = $object;
			}
		}
		return $ret;
	}
	/**
	***
	*** @NAME: getObject
	*** @DESC: Return plan object by object ID
	*** @IN: (INTEGER) $objId => ID of the object to fetch
	*** @OUT: Object of the plan. 'PlanObject' class type
	*** @THROW: /
	**/
	public function getObject($objId)
	{
		return $this->list_objects[$objId];
	}
}
?>
