<?php
/*
** @name: plan.lib.php
**
** @description: functions used for all plans types
** @author: Quentin George <q.george@esia-sa.com>
** @date: 31/03/2022
*/

include_once MODEL_PLAN;

/**
*** @name: setPlanNavbar
*** @desc: Create breadcrumbs navbar from plan root folder to current plan
*** @param PDO $dbconnection PDO object
*** @param object $navbar => navbar class from common.php
*** @param string $rootName => name of the plan plugin ($lang['WEBP_PLAN']) for the plan root link text
*** @param integer $userId => current user identifier
*** @param string $foldersURL => path to the folders main controller
*** @param object $plan => Plan.class instance of the current plan
**/
function setPlanNavbar($dbconnection, $navbar, $rootName, $userId, $foldersURL, $plan, $apache_path)
{
	// Add Plans Root folder to the navbar
	$navbar->addElement($rootName, $foldersURL);
	// Add folders
	if(!empty($plan->getFolderId()))
	{
		$navbarElements = array();
		$elemId = $plan->getFolderId();
		$folders = Model_plan::getFoldersTree($dbconnection, $userId, $plan->getFolderId());
		while(isset($folders[$elemId]))
		{
			array_unshift($navbarElements, array('name' => $folders[$elemId]['name'], 'href' => $foldersURL . '?id=' . $elemId));
			$elemId = $folders[$elemId]['parent'];
		}
		foreach($navbarElements as $navElt)
		{
			$navbar->addElement($navElt['name'], $navElt['href']);
		}
	}
	// Add plan
	$navbar->addElement($plan->getName(), $apache_path . $plan->getTypeController() . '&id=' . $plan->getId());
}
/**
*** @name: getMoveFoldersList
*** @desc: Get folders in wich the current user can move a plan or a folder
*** @param PDO $dbconnection PDO object
*** @param integer $userId current user identifier
*** @param string $rootName => name of the plan root folder from $lang $lang['PLAN_FOLDER_ROOT'])
*** @param integer $parentId => id of the parent folder of the current plan or folder
*** @param integer $folderId => id of the current element if it's a folder
*** @return array list of folders wich the user can move the current plan or folder inside
**/
function getMoveFoldersList($dbconnection, $userId, $rootName='Root', $parentId, $folderId=NULL)
{
	$moveList = array();

	// Get the list of folders wich the user has the write privilege on it
	$folders = Model_Plan::getFolders($dbconnection, $userId, NULL, NULL, TRUE);
	if(!empty($folderId))
	{
		// If the element to move is a folder, we must remove it's children from the list. Moving it inside one of it's children will break the tree.
		$folderChildren = Model_Plan::getFoldersTree($dbconnection, $userId, $folderId, Model_plan::PLAN_SORT_CHILDREN);
		$folders = array_diff_key($folders, $folderChildren);
	}
	if(!empty($parentId) && empty($folders[$parentId]))
	{
		// If parent isn't in the folders write list because the user hasn't the write right on it. In order to keep the current parent, we have to add it into the move list.
		$foldersParent = Model_Plan::getFolders($dbconnection, $userId, $parentId);
		$folders[$parentId] = $foldersParent[$parentId];
	}

	if(!empty($folders))
	{
		foreach($folders as $fold)
		{
			$id = $fold['id'];
			$name = $folders[$id]['name'];
			$pid = $fold['parent']; // Parent ID
			if (!empty($pid))
			{
				$pname = '...'; // Parent name
				if (!empty($folders[$pid]['name']))
				{
					$pname = $folders[$pid]['name'];
				}
				$name = $pname . ' > ' . $name;
				if (!empty($folders[$pid]['parent']))
				{
					$name = '... > ' . $name;
				}
			}
			$moveList[] = array('id' => $id, 'name' => $name);
		}
		// Sort list by name
		usort($moveList, function($a, $b) {
			return strcmp($a['name'], $b['name']);
		});
	}
	// Add root folder to the list
	array_unshift($moveList, array('id' => 0, 'name' => $rootName));

	return $moveList;
}
/**
*** @name: setPlanCommonData
*** @desc: Set plan class common data from update plan form
*** @param PDO $dbconnection PDO object
*** @param array $lang => lang var from common.php
*** @param object $plan => Plan.class instance of the current plan
*** @param array $data => form data from $data
*** @return object Plan.class Updated
*** @throws \EsiaDbException | \Plan\PlanException | \Exception
**/
function setPlanCommonData($dbconnection, $lang, $plan, $data)
{
	// Get users rights
	$usersListByAccess = Model_plan::getPlanAccessUsersList($dbconnection, $plan->getId());
	$plan->setAccessUsersList($usersListByAccess['READ'], $usersListByAccess['WRITE'], $usersListByAccess['CREATOR']);

	// Update plan general parameters
	if(!empty($data['plan_name']))
	{
		$plan->setName($data['plan_name']);
	}
	else
	{
		throw new \Exception($lang['PLAN_ERROR_PLAN_PARAM'] . ": " . $lang['PLAN_ERROR_PLAN_NAME']);
	}
	if(isset($data['folder_parent']))
	{
		$plan->setFolderId($data['folder_parent']);
	}

	// Update plan users access
	$usersRights = array(
		'READ' => array(),
		'WRITE' => array()
	);
	// If the multi chosen is empty the key does not exist in $data. If not set means that no users are selected
	if(isset($data['read_userplan']))
	{
		$usersRights['READ'] = $data['read_userplan'];
	}
	if(isset($data['write_userplan']))
	{
		$usersRights['WRITE'] = $data['write_userplan'];
	}
	$plan->setAccessUsersList($usersRights['READ'], $usersRights['WRITE'], NULL);

	// Update plan parameters refresh
	if(!empty($data['refresh']))
	{
		$plan->setParam(array('refresh'), $data['refresh']);
	}
	else
	{
		$plan->setParam(array('refresh'), 0);
	}

	// Plan size params
	if(!empty($data['plan_size']))
	{
		$planSize = json_decode($data['plan_size'], TRUE);

		if(is_numeric($planSize['width']) && is_numeric($planSize['height']))
		{
			$plan->setParam(array('size'), $planSize);
		}
	}

	return $plan;
}
/**
*** @name: getNodeServicesList
*** @desc: Get a list of all services of a device
*** @param PDO $dbconnection PDO object
*** @param array $lang => lang var from common.php
*** @param integer $nodeId => id of the device
*** @return array list of services id & name
*** @throws \EsiaDbException
**/
function getNodeServicesList($dbconnection, $lang, $nodeId)
{
	$servicesList = array();
	$services = Model_esia::getServices($dbconnection, $nodeId);
	foreach($services as $service)
	{
		$common_name = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);
		$servicesList[] = array(
			'id' => $service['id'],
			'name' => $common_name
		);
	}

	return $servicesList;
}
/**
*** @name: deleteFileErrorsHandler
*** @desc: Delete a file and return its name into an array if delete failed. Can be used to remove multiples files into a loop and keep all failed names into an array
*** @param string $file => complete file path and name of the file to delete
*** @param array $errors => list of files from previous files deletion errors
*** @return array list of files names which can't be deleted
**/
function deleteFileErrorsHandler($file, $errors=array())
{
	if(file_exists($file))
	{
		if(!unlink($file))
		{
			$errors[] = $file;
		}
	}

	return $errors;
}
/**
*** @name: rmdirRecursive
*** @desc: Delete a directory and all its content
*** @param string $dir => directory path
**/
function rmdirRecursive($dir)
{
	$content = array_diff(scandir($dir), array('..', '.'));
	foreach($content as $value)
	{
		$item = $dir . '/' . $value;
		if(is_dir($item))
		{
			// Directory
			rmdirRecursive($item);
		}
		else
		{
			// File
			@unlink($item);
		}
	}
	@rmdir($dir);
}
/**
*** @name: cleanCheckBoxesVals
*** @desc: Clean checkbox array vals to have only true or false (checkboxes return a value if checked and nothing(null) if not)
*** @param array $checkBoxes => associative array where key is check box name and value is the input value
*** @return array associative array where key is check box name and val is true if checked
**/
function cleanCheckBoxesVals($checkBoxes)
{
	foreach($checkBoxes as $key => $value)
	{
		if($value !== NULL)
		{
			$checkBoxes[$key] = TRUE;
		}
		else
		{
			$checkBoxes[$key] = FALSE;
		}
	}

	return $checkBoxes;
}
/**
*** @name: getNodeRrdList
*** @desc: Get a list of all values for all services of a device. Add selected tag TRUE for all values (service_id-value_name) passed in the array $objSelectedVal
*** @param PDO $dbconnection PDO object
*** @param array $lang => lang var from common.php
*** @param integer $nodeId => id of the device
*** @param array $objSelectedVal => List of values (service_id-value_name) to add selected tag to true
*** @return array list of services values (RRD) value, name, selected, order
*** @throws \EsiaDbException
**/
function getNodeRrdList($dbconnection, $lang, $nodeId, $objSelectedVal=array())
{
	$rrdList = array();
	$objSelectedValOrder = array_flip($objSelectedVal);
	$services = Model_esia::getServices($dbconnection, $nodeId);
	foreach($services as $service)
	{
		$common_name = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);
		$values = array_keys($service['value']);
		foreach($values as $val)
		{
			$value = $service['id'] . '-' . $val;
			$selected = FALSE;
			$order = NULL;
			if(in_array($value, $objSelectedVal))
			{
				$selected = TRUE;
				$order = $objSelectedValOrder[$value];
			}
			$rrdList[] = array(
				'value' => $value,
				'name' => $common_name . ' > ' . ucfirst($val),
				'selected' => $selected,
				'order' => $order
			);
		}
	}

	return $rrdList;
}
/**
*** @name: getEsiaPlanPerms
*** @desc: Get boolean for each webp plan esia permission by user.
*** @param PDO $dbconnection PDO object
*** @param Class $user => User class from common.php
*** @return array list of booleans, view and admin, true if the user has the permission
*** @throws \EsiaDbException
**/
function getEsiaPlanPerms($dbconnection, $user)
{
	$isUserPerms = array(
		'view' => ($user->getDroits($dbconnection, "map_view") == 1),
		'admin' => ($user->getDroits($dbconnection, "map_administration") == 1),
	);

	return $isUserPerms;
}
?>
