<?php
/*
** @name: plan_img.lib.php
**
** @description: functions used for plans type img
** @author: Quentin George <q.george@esia-sa.com>
** @date: 31/03/2022
*/

include_once MODEL_PLAN;

// Define constants and usefull variables specific to this plan type
define('VALID_BG_FILE_EXT', array('jpg', 'jpeg', 'bmp', 'gif', 'png'));
define('DEFAULT_BG_FILE', 'empty.jpg');

/**
*** @name: cleanUploadFileName
*** @desc: Clean file name to avoid unexpected chars like é,&,@,...
*** @param string $fileName => file name to check unexpected chars
*** @return string cleaned file name
**/
function cleanUploadFileName($fileName)
{
	return trim(preg_replace('/[^A-Za-z0-9-\.]+/', '-', $fileName));
}
/**
*** @name: checkUploadFilePhpErrors
*** @desc: Verify if an uploaded file has php errors
*** @param integer $fileError => php $_FILES error code ($_FILES['fileInputName']['error'])
*** @throws Exception error message string corresponding to php error code
**/
function checkUploadFilePhpErrors($fileError)
{
	if($fileError)
	{
		switch($fileError)
		{
			case 1: // UPLOAD_ERR_INI_SIZE
				throw new Exception('File exceeds the maximum size by server (php.ini)');
				break;
			case 2: // UPLOAD_ERR_FORM_SIZE
				throw new Exception('File exceeds the maximum size by HTML form');
				break;
			case 3: // UPLOAD_ERR_PARTIAL
				throw new Exception('File transfer was interrupted');
				break;
			case 4: // UPLOAD_ERR_NO_FILE
				throw new Exception('File size is null');
				break;
			default:
				throw new Exception('Unknown error');
		}
	}
}
/**
*** @name: uploadImgOptimization
*** @desc: Optimize uploaded image for web usage with transparency for supported image types
*** @param string $image => path to the uploaded image
*** @param string $fileExt => image type (png, bmp). There is no use of this function for svg files
*** @param integer (optional) $newHeight => new height if the image require to be resized. Width will be aspect ratio recalculated
**/
function uploadImgOptimization($image, $fileExt, $newHeight=0)
{
	// No use for svg files
	if($fileExt == 'svg')
	{
		return;
	}
	// Get original file sizes
	list($width, $height) = getimagesize($image);
	// Check if resize is required
	if($newHeight > 0)
	{
		$newHeight = round($newHeight);
		// Calc new width from height aspect ration
		$newWidth = round(($width / $height) * $newHeight);
	}
	else
	{
		$newHeight = $height;
		$newWidth = $width;
	}
	// Draw a new image
	$tmp_img = imagecreatetruecolor($width, $height);

	if($fileExt == 'png' || $fileExt == 'bmp' || $fileExt == 'gif')
	{
		// File types which allow transparency
		// Create img background
		$bg = imagecolorallocatealpha($tmp_img, 0, 0, 0, 127);
		imagecolortransparent($tmp_img, $bg);
		imagefill($tmp_img, 0, 0, $bg);
		// Create new src image to prepare resample
		switch($fileExt)
		{
			case 'png':
				$source = imagecreatefrompng($image);
				break;
			case 'gif':
				$source = imagecreatefromgif($image);
				break;
			case 'bmp':
				$source = imagecreatefrombmp($image);
				break;
		}
		// Resize and resample
		imagecopyresampled($tmp_img, $source, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
		// Save the image transparency after modifications
		imagesavealpha($tmp_img, TRUE);
		// Save the new image
		switch($fileExt)
		{
			case 'png':
				imagepng($tmp_img, $image);
				break;
			case 'gif':
				imagegif($tmp_img, $image);
				break;
			case 'bmp':
				imagebmp($tmp_img, $image);
				break;
		}
	}
	else
	{
		// JPG files (without transparency)
		// Create img background
		$bg = imagecolorallocate($tmp_img, 255, 255, 255);
		imagefill($tmp_img, 0, 0, $bg);
		// Create new src image to prepare resample
		$source = imagecreatefromjpeg($image);
		// Resize and resample
		imagecopyresampled($tmp_img, $source, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
		// Save the new image
		imagejpeg($tmp_img, $image);
	}
}
/**
*** @name: uploadNewIcon
*** @desc: Upload an icon for plan img admin menu icon
*** @param string $path => DOCUMENT_ROOT of the server
*** @param array $file => php $_FILES data for this icon
*** @param string $fileExt => image type (png, bmp). There is no use of this function for svg files
*** @param string $iconName => name of the icon
*** @param string $status => state of the icon (critical, warning, ok, unknown)
**/
function uploadNewIcon($path, $file, $fileExt, $iconName, $status)
{
	// Check upload php file errors
	checkUploadFilePhpErrors($file['error']);

	$upload_icon = $path . DATA_ICONS . $iconName . '_' . $status . '.' . $fileExt;

	// If file is not already present
	if(!file_exists($upload_icon))
	{
		$isMoveSuccess = move_uploaded_file($file['tmp_name'], $upload_icon);

		if($isMoveSuccess == TRUE)
		{
			// If image successfully uploaded resample the image. This function will do nothing for svg files
			uploadImgOptimization($upload_icon, $fileExt, 0);
		}
		else
		{
			throw new Exception('File not uploaded (icons)');
		}
	}
	else
	{
		throw new Exception('File name ' . $iconName . ' already exist. Choose another name.');
	}
}
?>
