<?php
/*
** @name: plan_rack.lib.php
**
** @description: functions used for plans type rack
** @author: Quentin George <q.george@esia-sa.com>
** @date: 31/03/2022
*/

include_once MODEL_PLAN;

// Valid types ex: server=item-type, none=item-subtype, 1,2,3,4=item-size
define('RACK_ITEMS_VALID_TYPES', array(
	"server" => array(
		"none" => array("1", "2", "3", "4")
	),
	"storage" => array(
		"none" => array("1", "2", "3", "4")
	),
	"switch" => array(
		"8" => array("1"),
		"16" => array("1"),
		"24" => array("1"),
		"48" => array("1")
	),
	"router" => array(
		"4" => array("1"),
		"8" => array("1")
	),
	"firewall" => array(
		"4" => array("1"),
		"8" => array("1")
	),
	"pdu" => array(
		"C13" => array("1")
	),
	"ups" => array(
		"none" => array("1", "2", "3", "4", "5")
	),
	"sensor" => array(
		"4" => array("1"),
		"8" => array("1")
	),
	"generic" => array(
		"none" => array("1", "2", "3", "4", "5")
	),
	"shelf" => array(
		"none" => array("1")
	),
	"cable_manag" => array(
		"tray" => array("1"),
		"rings" => array("1")
	),
	"patch_panel" => array(
		"ethernet" => array("1"),
		"fiber" => array("1")
	),
	"console" => array(
		"none" => array("1")
	),
	"power_strip" => array(
		"E" => array("1")
	),
	"blank" => array(
		"none" => array("1", "2", "3", "4")
	)
));
// Display states for each rack items
define('RACK_ITEMS_STATES_TO_DISPLAY', array(
	"server" => array("ping", "raid", "disks", "power", "temp", "fans"),
	"storage" => array("ping", "cpu", "raid", "disks", "temp"),
	"switch" => array("ping", "ifs"),
	"router" => array("ping", "cpu", "bandwidth", "connections", "ifs"),
	"firewall" => array("ping", "cpu", "bandwidth", "connections", "vpn", "ifs"),
	"pdu" => array("ping", "ifs", "outlets"),
	"ups" => array("ping", "power", "batteries", "health"),
	"sensor" => array("ping", "ifs", "inputs"),
	"generic" => array("ping", "cpu", "mem")
));
// Items states which can have multiple services
define('RACK_ITEMS_STATES_OPTS', array(
	"disks" => array(
		"isMulti" => TRUE,
		"plugins_tags" => array("DISK", "STORAGE")
	),
	"fans" => array(
		"isMulti" => TRUE,
		"plugins_tags" => array("FANS", "HEALTH")
	),
	"ifs" => array(
		"isMulti" => TRUE,
		"plugins_tags" => array("INTERFACE")
	),
	"inputs" => array(
		"isMulti" => TRUE,
		"plugins_tags" => array("SENSOR")
	),
	"ping" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("PING", "ICMP")
	),
	"cpu" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("CPU", "LOAD")
	),
	"mem" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("MEMORY", "MEM", "RAM")
	),
	"raid" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("RAID")
	),
	"power" => array(
		"isMulti" => TRUE,
		"plugins_tags" => array("POWER")
	),
	"temp" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("HEALTH", "TEMPERATURE")
	),
	"bandwidth" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("INTERFACE")
	),
	"connections" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("SESSIONS", "CONNECTIONS")
	),
	"vpn" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("VPN")
	),
	"outlets" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("OUTLET", "HEALTH"),
		"svg_variants" => array("E", "C13")
	),
	"batteries" => array(
		"isMulti" => FALSE,
		"plugins_tags" => array("BATTERY")
	),
	"health" => array(
		"isMulti" => TRUE,
		"plugins_tags" => array("STATE", "ALARM")
	)
));

/**
*** @name: isValidRackItem
*** @desc: Check if a rack item is valid (valid associations between type, subtype and u-size)
*** @param string $type => rack object type (example => type: switch)
*** @param string $subType => rack object sub_type (example => type: switch, sub_type: 4)
*** @param integer $size => size in number of U
*** @return boolean if valid TRUE
**/
function isValidRackItem($type, $subType, $size)
{
	if(isset(RACK_ITEMS_VALID_TYPES[$type][$subType]) && in_array($size, RACK_ITEMS_VALID_TYPES[$type][$subType]))
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}
/**
*** @name: formatNodeServicesList
*** @desc: Get a list of all services of a device
*** @param PDO $dbconnection PDO object
*** @param array $lang => lang variable from common.php
*** @param integer $nodeId => node identifier
*** @return array list of services informations
**/
function formatNodeServicesList($dbconnection, $lang, $nodeId)
{
	$servicesList = array();
	$services = Model_esia::getServices($dbconnection, $nodeId);
	foreach ($services as $service)
	{
		$common_name = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);

		$servicesList[$service['id']] = array(
			'id' => $service['id'],
			'name' => $common_name,
			'plugin_id' => $service['plugin_id'],
			'plugin_name' => $service['check_plugin_name']
		);
	}

	return $servicesList;
}
/**
*** @name: getRackItemsUlinkAndDeviceName
*** @desc: Get plan objects devices name & for each rack U the ID of the object linked to it
*** @param PDO $dbconnection PDO object
*** @param array $objectsList => list of plan objects as ObjectPlan.class
*** @return array list of services informations
**/
function getRackItemsUlinkAndDeviceName($dbconnection, $objectsList)
{
	$uLinksIds = array();
	$devicesName = array();
	foreach($objectsList as $objId => $object)
	{
		// Get device name for rack item objects
		if($object->getTypeName() == 'rack_item')
		{
			$deviceId = 0;
			$params = $object->getParams();
			if(isset($params['config']['device_id']) && is_numeric($params['config']['device_id']))
			{
				$deviceId = $params['config']['device_id'];
			}
			if(!empty($deviceId))
			{
				$devicesName[$objId] = Model_plan::getDeviceNameById($dbconnection, $deviceId);
			}
			else
			{
				$devicesName[$objId] = "";
			}

			// Get all U occupied numbers and get the objects ID linked to it
			if(isset($params['common']['u_size']) && isset($params['common']['u_position']))
			{
				$i = 1;
				while($i <= intval($params['common']['u_size']))
				{
					$uLinksIds[intval($params['common']['u_position']) - ($i - 1)] = $objId;
					$i++;
				}
			}
		}
	}

	return array('uLinksIds' => $uLinksIds, 'devicesName' => $devicesName);
}
?>
